########
# Author:        ajb
# Maintainer:    $Author: gq1 $
# Created:       2008-09-26
# Last Modified: $Date: 2009-01-21 16:02:20 +0000 (Wed, 21 Jan 2009) $
# Id:            $Id: cache_creator.pm 4034 2009-01-21 16:02:20Z gq1 $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/trunk/lib/npg_qc/api/cache_creator.pm $
#
package npg_qc::api::cache_creator;
use strict;
use warnings;
use Class::Std;
use Carp;
use English qw{-no_match_vars};
use base qw(npg_qc::api::base);
use Readonly;

our $VERSION = do { my ($r) = q$LastChangedRevision: 4034 $ =~ /(\d+)/mxs; $r; };

Readonly our $TRIES => 10;
Readonly our $TRY_SUCCESS => -1;
Readonly our $TEMP_ID_RUN => 5;
Readonly our $MINIMUM_ID_RUN => 1;
1;

{

  ## no critic (ProhibitUnusedVariables)
  my %response_statement_of :ATTR( :get<response_statement>, :set<response_statement> );
  ## use critic

  sub send_request_to_run_cache_query {
    my ($self, $id_run) = @_;
    my $util = $self->util();

    my $xml = q{<?xml version="1.0" encoding="utf-8"?><root><method>cache_query</method>};
    if ($id_run) {
      $xml .= qq{<id_run>$id_run</id_run>};
    }
    $xml .= q{</root>};

    my $response = $util->post(
      $util->base_uri().q{cache_query},
      [
        'Content_Type'   => 'application/xml',
  			'Content_Length' => length $xml,
  			'Content'        => $xml,
  			'Accept'         => 'text/xml',
			]
    );

    if ($response->is_success()) {
      my $parser = $self->util->parser();
      my ($statement) = $parser->parse_string($response->content)->getElementsByTagName('response') =~ /([A-Za-z]+)/xms;

      $self->set_response_statement($statement);

      if ($statement ne 'success') {
        croak 'Response not good';
      }

    } else {
      $self->set_response_statement('no success');
      croak 'failed to successfully post - this could be from a timeout issue';
    }

    return 1;
  }

}
1;
__END__

=head1 NAME

npg_qc::api::cache_creator

=head1 VERSION

$LastChangedRevision: 4034 $

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 new - Constructor

  my $oCacheCreator = npg_qc::api::cache_creator->new({'util' => $oUtil,});

  my $oCacheCreator = npg_qc::api::cache_creator->new({
    # 'key' => value parameters for the derived class
  });

=head2 get_response_statement - returns a response statement dependent on statement sent back from web page, or croaking

  my $sResponseStatement = $oCacheCreator->get_response_statement();

=head2 set_response_statement - sets the response statement into the object

  $oCacheCreator->set_response_statement($statment);

=head2 send_request_to_run_cache_query - fires off the web request to cache queries. Croaks or returns true

  eval {
    $boolSendRequestToRunCacheQuery = $oCacheCreator->send_request_to_run_cache_query();
  } or do {
    <code to check why croaked>
  };

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

strict
warnings
Class::Std
Carp
English
npg_qc::api::base
Readonly

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Andy Brown, E<lt>ajb@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2008 GRL, by Andy Brown

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.4 or,
at your option, any later version of Perl 5 you may have available.

=cut
