#########
# Author:        gq1
# Maintainer:    $Author: gq1 $
# Created:       2000-02-12
# Last Modified: $Date: 2009-02-19 15:22:16 +0000 (Thu, 19 Feb 2009) $
# Id:            $Id: delete_qc_data.pm 4264 2009-02-19 15:22:16Z gq1 $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/trunk/lib/npg_qc/api/delete_qc_data.pm $
#
package npg_qc::api::delete_qc_data;
use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use base qw(npg_qc::api::base);
use Readonly;

our $VERSION = do { my ($r) = q$LastChangedRevision: 4264 $ =~ /(\d+)/mxs; $r; };


Readonly our @TILE_BASED_TABLES => qw(
                                    cumulative_errors_by_cycle
                                    error_rate_reference_including_blanks
                                    error_rate_reference_no_blanks
                                    error_rate_relative_reference_cycle_nucleotide
                                    error_rate_relative_sequence_base
                                    errors_by_cycle
                                    errors_by_cycle_and_nucleotide
                                    errors_by_nucleotide
                                    information_content_by_cycle
                                    lane_qc
                                    log_likelihood
                                    most_common_blank_pattern
                                    most_common_word
                                    tile_all
                                    tile_score                                    
                                    );
Readonly our @RUN_BASED_TABLES  => qw(
                                    analysis_lane
                                    analysis
                                    chip_summary
                                    frequency_response_matrix
                                    image_store
                                    instrument_statistics
                                    offset
                                    qcal
                                    run_config
                                    signal_mean
                                    );
{

  ##no critic (ProhibitUnusedVariables)
  my %id_run_of         :ATTR( :get<id_run>,          :set<id_run>         );
  my %dbh               :ATTR( :get<dbh>,             :set<dbh>            );
  ##use critic

  sub id_run {
    my ($self, $id_run) = @_;
    if ($id_run) {
      $self->set_id_run($id_run);
    }
    return $self->get_id_run();
  }

  sub dbh {
    my ($self, $dbh) = @_;
    if ($dbh) {
      $self->set_dbh($dbh);
    }
    if (!$self->get_dbh()) {
      $dbh = $self->util->dbh();
      $self->set_dbh($dbh);
    }
    return $self->get_dbh();
  }

  sub delete_tile_based_data {
    my ($self, $table_name) = @_;
    
    my $query = qq{DELETE $table_name
                   FROM $table_name, run_tile
                   WHERE $table_name.id_run_tile = run_tile.id_run_tile
                   AND run_tile.id_run = ?
                   };

    my $dbh = $self->dbh();
    my $rows_deleted;
    eval {
      $rows_deleted = $dbh->do($query, {}, $self->get_id_run());
      print "$rows_deleted rows deleted from table $table_name\n"
      or croak 'cannot print';
      #$dbh->commit();
      1;
    } or do {
      $dbh->rollback();
      $dbh->disconnect();
      croak "Error happened to delete data from table $table_name:\n$EVAL_ERROR";
    };

    return $rows_deleted;
  }
  
  sub delete_all_tile_based_data {
    my ($self) = @_;

    foreach my $table_name (@TILE_BASED_TABLES){
      print ">trying to delete data from table $table_name\n"
      or croak 'cannot print';
      $self->delete_tile_based_data($table_name);
    }

    return 1;
  }

  sub delete_run_based_data {
    my ($self, $table_name) = @_;
    
    my $query = qq{DELETE from $table_name
                   WHERE id_run = ?
                   };

    my $dbh = $self->dbh();
    my $rows_deleted;
    eval {
      $rows_deleted = $dbh->do($query, {}, $self->get_id_run());
      print "$rows_deleted rows deleted from table $table_name\n"
      or croak 'cannot print';
      #$dbh->commit();
      1;
    } or do {
      $dbh->rollback();
      $dbh->disconnect();
      croak "Error happened to delete data from table $table_name:\n$EVAL_ERROR";
    };

    return $rows_deleted;
  }

  sub delete_all_run_based_data {
    my ($self) = @_;

    foreach my $table_name (@RUN_BASED_TABLES){
      print ">trying to delete data from table $table_name\n"
      or croak 'cannot print';
      $self->delete_run_based_data($table_name);
    }

    return 1;
  }

  sub delete_run_based_data_from_sql{
    my ($self, $sql) = @_;

    my $dbh = $self->dbh();
    my $rows_deleted;
    eval {
      $rows_deleted = $dbh->do($sql, {});
      print "$rows_deleted rows deleted based on SQL:\n$sql\n"
      or croak 'cannot print';
      #$dbh->commit();
      1;
    } or do {
      $dbh->rollback();
      $dbh->disconnect();
      croak "Error happened to delete data:\n$EVAL_ERROR";
    };

    return $rows_deleted;
  }
  
  sub delete_other_data{
    my ($self) = @_;

    my $id_run = $self->get_id_run();

    my $sql = qq{DELETE analysis_lane_qcal FROM analysis_lane_qcal, analysis_lane
                WHERE analysis_lane.id_analysis_lane = analysis_lane_qcal.id_analysis_lane
                AND analysis_lane.id_run = $id_run};
    print ">trying delete data from table analysis_lane_qcal\n" or croak 'cannot print';
    $self->delete_run_based_data_from_sql($sql);
    
    $sql = qq{DELETE FROM run_graph where id_run = $id_run
                OR id_run = (SELECT id_run_pair FROM run_and_pair WHERE id_run = $id_run);};
    print ">trying delete data from table run_graph\n" or croak 'cannot print';
    $self->delete_run_based_data_from_sql($sql);
    
    $sql = qq{DELETE FROM cache_query WHERE type IN ('lane_summary', 'cycle_count') AND id_run = $id_run};
    print ">trying delete data from table cache_query\n" or croak 'cannot print';
    $self->delete_run_based_data_from_sql($sql);
    
    return 1;
  }

  sub delete_all_qc_data{
    my ($self) = @_;
    if($self->util()->dbsection() eq 'live'){
      print "**********\nLIVE DATABASE IS BEING USED!!!\n**********\n" or croak 'cannot print';
    }

    $self->delete_other_data();
    $self->delete_all_tile_based_data();
    $self->delete_all_run_based_data();
    $self->dbh()->commit();
    $self->dbh()->disconnect();

    return 1;
  }
}

1;
__END__

=head1 NAME

npg_qc::api::delete_qc_data

=head1 VERSION

$LastChangedRevision: 4264 $

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 dbh  - accessor method for dbh

=head2 id_run - accessor method  for id_run

=head2 delete_tile_based_data - given one table linked to run_tile table, delete data for this run

=head2 delete_all_tile_based_data - based on a list of tables related to run_tile, delete data from them

=head2 delete_run_based_data -given a talbe name, each row with id_run as a field, delete the data for this run

=head2 delete_all_run_based_data guveb a list of tables related to id_run, delete the data for this run

=head2 delete_run_based_data_from_sql - given a sql, try to excute it to delete data relate this run

=head2 delete_other_data - given three sql statements to delete data from table analysis_lane_qcal, run_graph and cache_query

=head2 delete_all_qc_data - delete all the QC data related to one run

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

strict
warnings
Carp
English
npg_qc::api::base


=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Guoying Qi, E<lt>gq1@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2008 GRL, by Guoying Qi

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.4 or,
at your option, any later version of Perl 5 you may have available.

=cut
