#########
# Author:        gq1
# Maintainer:    $Author: gq1 $
# Created:       2009-01-07
# Last Modified: $Date: 2009-01-22 14:59:44 +0000 (Thu, 22 Jan 2009) $
# Id:            $Id: matrix.pm 4047 2009-01-22 14:59:44Z gq1 $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/trunk/lib/npg_qc/api/loader/matrix.pm $
#

package npg_qc::api::loader::matrix;

use strict;
use warnings;
use Class::Std;
use Carp qw(carp croak confess cluck);
use English qw{-no_match_vars};
use base qw(npg_qc::api::loader);

our $VERSION = do { my ($r) = q$LastChangedRevision: 4047 $ =~ /(\d+)/mxs; $r; };

{

  sub read_xml {
    my ($self) = @_;

    my $parser = $self->util->parser();
    my $parsed_xml = $parser->parse_string($self->get_xml());

    my $response = $parsed_xml->getElementsByTagName('frequency_responses')->[0];
    
    my $id_run = $response->getAttribute('id_run');
    my $position = $response->getAttribute('position');
    my $cycle = $response->getAttribute('cycle');
    
    my @response_list = $parsed_xml->getElementsByTagName('response');

    foreach my $response (@response_list) {

      eval {
        $self->save_one_response($id_run, $cycle, $position, $response);
        1;
      } or do {
        $self->dbh->disconnect();
        croak $EVAL_ERROR;
      };
    }
    $self->dbh->commit();
    return 1;
  }

  sub save_one_response{
    my ($self, $id_run, $cycle, $position, $response) = @_;

    my $base = $response->getAttribute('base');
    my $red1 = $response->getAttribute('red1');
    my $red2 = $response->getAttribute('red2');
    my $green1 = $response->getAttribute('green1');
    my $green2 = $response->getAttribute('green2');

    my $query = q{INSERT INTO frequency_response_matrix
                  (id_run, cycle, lane, base, red1, red2, green1, green2) 
                  VALUES (?,?,?,?,?,?,?, ?)};
    eval{
      $self->dbh->do($query, {}, $id_run, $cycle, $position, $base, $red1, $red2, $green1, $green2);

      1;
    } or do {
	     if ($EVAL_ERROR =~ /Duplicate\ entry/xms) {
	        #carp $EVAL_ERROR. q{ - continuing};
	     } else {
	        $self->dbh->rollback();
	        croak $EVAL_ERROR;
	    }
    };

    return 1;
  }
}
1;
__END__

=head1 NAME

npg_qc::api::loader::matrix

=head1 VERSION

$LastChangedRevision: 4047 $

=head1 SYNOPSIS

  my $oQCal = npg_qc::api::loader::matrix->new({util => $oUtil, xml => $matrix_xml_string});

=head1 DESCRIPTION

Loads the XML format of the matrix files into the database directly

=head1 SUBROUTINES/METHODS

=head2 read_xml - reads the tile_score_xml_string, and then saves the data out to the tables

=head2 save_one_response

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

strict
warnings
Class::Std
English
Carp
npg_qc::api::base

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Guoying Qi, E<lt>gq1@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2008 GRL, by Andy Brown

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.4 or,
at your option, any later version of Perl 5 you may have available.

=cut
