#########
# Author:        gq1
# Maintainer:    $Author: gq1 $
# Created:       2009-01-09
# Last Modified: $Date: 2009-01-22 14:59:44 +0000 (Thu, 22 Jan 2009) $
# Id:            $Id: offset.pm 4047 2009-01-22 14:59:44Z gq1 $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/trunk/lib/npg_qc/api/loader/offset.pm $
#

package npg_qc::api::loader::offset;

use strict;
use warnings;
use Class::Std;
use Carp qw(carp croak confess cluck);
use English qw{-no_match_vars};
use base qw(npg_qc::api::loader);

our $VERSION = do { my ($r) = q$LastChangedRevision: 4047 $ =~ /(\d+)/mxs; $r; };

{

  sub read_xml {
    my ($self) = @_;

    my $parser = $self->util->parser();
    my $parsed_xml = $parser->parse_string($self->get_xml());

    my $offset_list = $parsed_xml->getElementsByTagName('offset_list')->[0];
    
    my $id_run = $offset_list->getAttribute('id_run');
    my $position = $offset_list->getAttribute('position');
    carp $id_run, $position;
    
    my @offsets = $parsed_xml->getElementsByTagName('offset');
    foreach my $offset (@offsets) {

      eval {
        $self->save_one_offset($id_run, $position, $offset);
        1;
      } or do {
        $self->dbh->rollback();
        $self->dbh->disconnect();
        croak $EVAL_ERROR;
      };
    }
    $self->dbh->commit();

    return 1;
  }

  sub save_one_offset{
    my ($self, $id_run, $position, $offset) = @_;

    my $tile = $offset->getAttribute('tile');
    my $cycle = $offset->getAttribute('cycle');
    my $image = $offset->getAttribute('image');
    my $x = $offset->getAttribute('x');
    my $y = $offset->getAttribute('y');

    my $query = q{INSERT INTO offset
                  (id_run, lane, tile, cycle, image, x, y) 
                  VALUES (?,?,?,?,?,?, ?)};
    eval{
      $self->dbh->do($query, {}, $id_run, $position, $tile, $cycle, $image, $x, $y);

      1;
    } or do {
	     if ($EVAL_ERROR =~ /Duplicate\ entry/xms) {
	        #carp $EVAL_ERROR. q{ - continuing};
	     } else {
	        $self->dbh->rollback();
	        croak $EVAL_ERROR;
	    }
    };

    return 1;
  }
}
1;
__END__

=head1 NAME

npg_qc::api::loader::offset

=head1 VERSION

$LastChangedRevision: 4047 $

=head1 SYNOPSIS

  my $oQCal = npg_qc::api::loader::offset->new({util => $oUtil, xml => $offset_xml_string});

=head1 DESCRIPTION

Loads the XML format of the offset files into the database directly

=head1 SUBROUTINES/METHODS

=head2 read_xml - reads the tile_score_xml_string, and then saves the data out to the tables

=head2 save_one_offset

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

strict
warnings
Class::Std
English
Carp
npg_qc::api::base

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Guoying Qi, E<lt>gq1@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2008 GRL, by Andy Brown

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.4 or,
at your option, any later version of Perl 5 you may have available.

=cut
