#########
# Author:        ajb
# Maintainer:    $Author: gq1 $
# Created:       2008-06-10
# Last Modified: $Date: 2009-01-22 14:59:44 +0000 (Thu, 22 Jan 2009) $
# Id:            $Id: summary_data.pm 4047 2009-01-22 14:59:44Z gq1 $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/trunk/lib/npg_qc/api/loader/summary_data.pm $
#

package npg_qc::api::loader::summary_data;

use strict;
use warnings;
use Class::Std;
use Carp qw(carp croak confess cluck);
use English qw{-no_match_vars};
use base qw(npg_qc::api::loader);

our $VERSION = do { my ($r) = q$LastChangedRevision: 4047 $ =~ /(\d+)/mxs; $r; };

{
  ## no critic (ProhibitUnusedVariables)
  my %chip_summary_xml_of :ATTR( :get<chip_summary_xml>, :set<chip_summary_xml> );
  my %lane_qc_xmls_of     :ATTR( :get<lane_qc_xmls>,     :set<lane_qc_xmls>     );
  ## use critic

  sub read_xml {
    my ($self) = @_;

    my $parser = $self->util->parser();
    my $parsed_xml = $parser->parse_string($self->get_xml());

    my $run = $parsed_xml->getElementsByTagName('run')->shift();
    $self->set_id_run($run->getAttribute('id_run'));
    my $dbh = $self->dbh();
    eval {
      $self->save_chip_summary($parsed_xml);
      $self->process_lane_qc($parsed_xml);
      1;
    } or do {
      $dbh->disconnect();
      croak $EVAL_ERROR;
    };
    $dbh->commit();
    return 1;
  }

  sub save_chip_summary {
    my ($self, $parsed_xml) = @_;

    my $run    = $parsed_xml->getElementsByTagName('run')->shift();
    my $paired = $run->getAttribute('paired');

    my $chip_summary     = $parsed_xml->getElementsByTagName('chip_summary')->shift();
    my $chip_summary_row = $chip_summary->getElementsByTagName('row')->shift();

    my $chip_id     = $chip_summary_row->getAttribute('chip_id');
    my $clusters    = $chip_summary_row->getAttribute('clusters');
    my $clusters_pf = $chip_summary_row->getAttribute('clusters_pf');
    my $yield_kb    = $chip_summary_row->getAttribute('yield_kb');
    my $machine     = $chip_summary_row->getAttribute('machine');
    
    my $query = q{INSERT into chip_summary(id_run,paired,chip_id,clusters,clusters_pf,yield_kb,machine)
                  VALUES (?,?,?,?,?,?,?)};

    my $dbh = $self->dbh();
    eval {
      $dbh->do($query,
                {}, $self->get_id_run(), $paired, $chip_id,$clusters,$clusters_pf,$yield_kb,$machine);
      $dbh->commit();
      1;
    } or do {
      if ($EVAL_ERROR =~ /Duplicate\ entry/xms) {
        #carp "$EVAL_ERROR - continuing anyway";
      } else {
        $self->dbh->rollback();
        croak $EVAL_ERROR;
      }
    };

    return 1;
  }

  sub process_lane_qc {
    my ($self, $parsed_xml) = @_;
    
    my $lane_qc = $parsed_xml->getElementsByTagName('lane_info')->shift();

    my @rows = $lane_qc->getElementsByTagName('row');
    my $dbh = $self->dbh();
    foreach my $row (@rows) {
      my $id_run = $self->get_id_run();
      my $lane   = $row->getAttribute('lane');
      my $tile   = $row->getAttribute('tile');
      my $end    = $row->getAttribute('end');

      my $args = {
        id_run => $id_run,
	lane   => $lane,
	tile   => $tile,
	end    => $end
      };
      
      my $id_run_tile = $self->run_tile($args)->{id_run_tile};

      my $clusters_raw           = $row->getAttribute('clusters_raw');
      my $av_1st_cycle_int_pf    = $row->getAttribute('av_1st_cycle_int_pf');
      my $av_perc_intensity_after_20_cycles_pf = $row->getAttribute('av_perc_intensity_after_20_cycles_pf');
      my $perc_pf_clusters       = $row->getAttribute('perc_pf_clusters');
      my $perc_align_pf          = $row->getAttribute('perc_align_pf');
      my $av_alignment_score_pf  = $row->getAttribute('av_alignment_score_pf');
      my $perc_error_rate_pf     = $row->getAttribute('perc_error_rate_pf');

      if ($perc_align_pf         eq 'unknown') { $perc_align_pf         = undef; };
      if ($av_alignment_score_pf eq 'unknown') { $av_alignment_score_pf = undef; };
      if ($perc_error_rate_pf    eq 'unknown') { $perc_error_rate_pf    = undef; };

      eval {
        $dbh->do(q{INSERT INTO lane_qc  (id_run_tile,end,clusters_raw,av_1st_cycle_int_pf,av_perc_intensity_after_20_cycles_pf,perc_pf_clusters,perc_align_pf,av_alignment_score_pf,perc_error_rate_pf)
        VALUES (?,?,?,?,?,?,?,?,?)}, {},
	$id_run_tile, $end, $clusters_raw,$av_1st_cycle_int_pf,$av_perc_intensity_after_20_cycles_pf,
	$perc_pf_clusters,$perc_align_pf,$av_alignment_score_pf,$perc_error_rate_pf);
	1;
      } or do {
        if ($EVAL_ERROR =~ /Duplicate\ entry/xms) {
         # carp "$EVAL_ERROR - continuing anyway";
        } else {
          $self->dbh->rollback();
          croak $EVAL_ERROR;
          
        }
      };
    }
    $dbh->commit();
    return 1;
  }

}
1;
__END__

=head1 NAME

npg_qc::api::loader::summary_data

=head1 VERSION

$LastChangedRevision: 4047 $

=head1 SYNOPSIS

  my $oLoaderSummaryData = npg_qc::api::loader::summary_data->new({util => $oUtil, xml => $summary_data_xml_string});

=head1 DESCRIPTION

Loads the XML format of the summary_data file into the database directly

=head1 SUBROUTINES/METHODS

=head2 read_xml - reads the summary_data_xml_string, and then saves the data out to the tables

=head2 save_chip_summary - deals with saving chip summary part to the chip summary table

=head2 process_lane_qc - deals with saving out all the lane_qc data to the lane_qc table

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

strict
warnings
Class::Std
English
Carp
npg_qc::api::base

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Andy Brown, E<lt>ajb@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2008 GRL, by Andy Brown

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.4 or,
at your option, any later version of Perl 5 you may have available.

=cut
