#########
# Author:        ajb
# Maintainer:    $Author: gq1 $
# Created:       2008-06-11
# Last Modified: $Date: 2009-03-20 10:55:56 +0000 (Fri, 20 Mar 2009) $
# Id:            $Id: main.pm 4573 2009-03-20 10:55:56Z gq1 $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/trunk/lib/npg_qc/api/main.pm $
#
package npg_qc::api::main;
use strict;
use warnings;
use Class::Std;
use Carp;
use English qw{-no_match_vars};
use base qw(npg_qc::api::base);
use Perl6::Slurp;
use npg_qc::api::process::bustard_output;
use npg_qc::api::run_tile;
use npg_qc::api::summary;
use srpipe::util;
use Readonly;
use npg_qc::api::process::runlog_z3;
use npg_qc::api::run_caching;
use npg_qc::api::loader::qcal;
use npg_qc::api::loader::run_config;
use npg_qc::api::loader::signal_mean;
use npg_qc::api::loader::summary_data;
use npg_qc::api::loader::tile_score;
use npg_qc::api::loader::matrix;
use npg_qc::api::loader::offset;

our $VERSION = do { my ($r) = q$LastChangedRevision: 4573 $ =~ /(\d+)/mxs; $r; };

Readonly our $TRIES => 10;
Readonly our $TRY_SUCCESS => -1;
Readonly our $TEMP_ID_RUN => 5;
Readonly our $MINIMUM_ID_RUN => 1;
Readonly our $MINIMUM_NUMBER_OF_EXPECTED_TILE_SCORE_XML_STRINGS => 200;

Readonly our %RUNS_TO_SKIP => ();

{
  ## no critic (ProhibitUnusedVariables)
  #a list of RunLog_concat.xml files in outgoing or analysis dir for each run
  my %runlog_file_urls_of     :ATTR( :get<runlog_file_urls>, :set<runlog_file_urls> );
  #a list of xml string with move_z information extracting from RunLog_concat.xml file
  my %runlog_output_of        :ATTR( :get<runlog_output>, :set<runlog_output> );
  #a list of run-tile from database, with field row not null
  my %runs_done_runlog_of     :ATTR( :get<runs_done_runlog>, :set<runs_done_runlog> );
  my %runlog_file_root_of     :ATTR( :get<runlog_file_root>, :set<runlog_file_root>);
  my %runlog_file_outgoing_of :ATTR( :get<runlog_file_outgoing>, :set<runlog_file_outgoing>);
  my %runlog_file_analysis_of :ATTR( :get<runlog_file_analysis>, :set<runlog_file_analysis>);
  my %srpipe_util_of          :ATTR( :get<srpipe_util>, :set<srpipe_util>);
  my %bustard_file_urls_of :ATTR( :get<bustard_file_urls>, :set<bustard_file_urls> );
  my %bustard_xml_output_of    :ATTR( :get<bustard_output>, :set<bustard_output> );
  my %response_objects_of  :ATTR( :get<response_objects>, :set<response_objects> );
  my %runs_of              :ATTR( :get<runs>, :set<runs> );
  my %bustard_runs_done_of :ATTR( :get<bustard_runs_done>, :set<bustard_runs_done> );
  my %file_root_of         :ATTR( :get<file_root>, :set<file_root> );
  my %file_dir_of          :ATTR( :get<file_dir>, :set<file_dir> );
  my %file_ext_of          :ATTR( :get<file_ext>, :set<file_ext> );
  my %signal_mean_dir_of       :ATTR( :get<signal_mean_dir>, :set<signal_mean_dir> );
  my %summary_dir_of       :ATTR( :get<summary_dir>, :set<summary_dir> );
  my %score_dir_of         :ATTR( :get<score_dir>, :set<score_dir> );
  my %fastqcheck_dir_of    :ATTR( :get<fastqcheck_dir>, :set<fastqcheck_dir> );
  my %matrix_dir_of        :ATTR( :get<matrix_dir>, :set<matrix_dir> );
  my %matrix_dir13_of        :ATTR( :get<matrix_dir13>, :set<matrix_dir13> );
  my %offset_dir_of        :ATTR( :get<offset_dir>, :set<offset_dir> );
  my %runs_finished        :ATTR( :get<runs_finished>, :set<runs_finished> );
  my %run_pairs            :ATTR( :get<run_pairs>, :set<run_pairs> );
  my %run_cache_of         :ATTR( :get<run_cache_object>, :set<run_cache_object> );
  my %run_folder_of        :ATTR( :get<run_folder>, :set<run_folder> );
  ## use critic

  #for process runlog.xml files
  #############################

  #get a list of run_tiles in the database
  sub runs_done_runlog {
    my ($self) = @_;

    print "checking any run in the database with run_tiles with row numbers\n" or croak "can't print\n";
    my $util = $self->util();
    my $run_tile_obj = npg_qc::api::run_tile->new({util => $util});
    my $movez_id_runs = $run_tile_obj->movez_id_runs();
    if($movez_id_runs){
      my $num_run_in_db = scalar keys %{$movez_id_runs};
      print "there are $num_run_in_db runs in the database with number of tiles which rows not null\n"
      or croak "can't print\n";
      #print  "The list of id_runs in the database with number of tiles which rows not null\n"
      #or croak "can't print\n";

     # foreach my $id_run (keys %{$movez_id_runs}) {
     #   my $num_tiles = $movez_id_runs-> {$id_run};
     #  print "$id_run $num_tiles\n" or croak "can't print\n";
     # }
      $self->set_runs_done_runlog($movez_id_runs);
    }else{
      $self->set_runs_done_runlog({});
      print "there is no id_run in the database with row not null\n" or croak "can't print\n";
    }
    return;
  }
  
  sub retrieve_runlogfile_urls {
    my ($self) = @_;
    #check any runlog already in the database
    $self->runs_done_runlog();

    #check any runs with runlog to do        
    $self->set_runlog_file_urls([]);

    my $srpipe_util = srpipe::util->new();
    #$self->set_file_root('t/data');   
    #my $runlog_outgoing = 'staging/IL*/outgoing/*/RunLog_concat.xml';
    #my $runlog_analysis = 'staging/IL*/analysis/*/RunLog_concat.xml';
    
    my @runlog_files_outgoing
      = $srpipe_util->my_glob($self->get_runlog_file_root(), $self->get_runlog_file_outgoing());
    
    my @runlog_files_analysis
      = $srpipe_util->my_glob($self->get_runlog_file_root(), $self->get_runlog_file_analysis());
         
	 foreach my $runlog_file (@runlog_files_outgoing, @runlog_files_analysis){
	   if($runlog_file !~ /xml$/mixs){
	     print "  ignore $runlog_file\n" or croak "can't print\n";
	     next;
	   }
      my ($id_run) = $runlog_file =~ /_0*(\d+)\/RunLog_concat/xms ;
      print "Getting id_run from the file $runlog_file: $id_run\n" or croak "can't print\n";
      if($id_run <= $TEMP_ID_RUN){
        print "  ignore $runlog_file\n" or croak "can't print\n";
        next;
      }
      my $num_tiles_done = $self->get_runs_done_runlog()->{$id_run};
      if ( $num_tiles_done ){
          print "  already done\n" or croak "can't print\n";
          next;
      }

      push @{$self->get_runlog_file_urls()}, $runlog_file;
    }
    my $has_files_to_process = scalar @{$self->get_runlog_file_urls()};
    print "$has_files_to_process runlog files need to be processed\n" or croak "can't print\n";
    return $has_files_to_process;
  }
 
  sub retrieve_runlog_xml {
    my ($self) = @_;
    my $util = $self->util();
    foreach my $file_url (@{$self->get_runlog_file_urls()}) {
      print "extracting move_zes from the file: $file_url\n" or croak "can't print\n";
      my $runlog_output = npg_qc::api::process::runlog_z3->new({util => $util});
 
      my ($id_run) = $file_url =~ /_0*(\d+)\/RunLog_concat/xms ;

      if (!$self->get_runlog_output()){
        $self->set_runlog_output([]);
      }
      my $xml_list;
      eval{
        $xml_list = $runlog_output->extract_movez($file_url);
        1;
      } or do {
        carp $EVAL_ERROR;
        carp "failed to extract data from $file_url";
      };

      my $num_tiles_in_log = $runlog_output->get_num_tiles_run();
      my $num_tiles_done = $self->get_runs_done_runlog()->{$runlog_output->get_id_run()};

      if(! defined $num_tiles_done){
        $self->set_runlog_output([]);
        if ($xml_list) {
          push @{$self->get_runlog_output()}, @{$xml_list};
        }
      
        $self->websend_runlog_xml();

        #caching number of movez_tiles out of range        
        my $api_run_caching = npg_qc::api::run_caching->new({util => $util});
        print " caching number of movez tiles out of range\n" or croak "can't print\n";
        $api_run_caching->cache_query_movez_tiles($id_run, 1);

      }
    }

    return;
  }

  sub websend_runlog_xml {
    my ($self) = @_;
    print "sending XML to database by web posting\n" or croak "can't print\n";
    my $util = $self->util();
    my $count_sent = 0;
    foreach my $xml (@{$self->get_runlog_output()}) {
      $count_sent++;
      print "sending $count_sent\n" or croak "can't print\n";
      #print $xml, "\n\n";
      my $return_value = $self->websend_any_xml($xml);
      if(!$return_value){
        croak "failed to sending $count_sent\n";
      }
      print "finish sending $count_sent\n" or croak "can't print\n";
    }
    return;
  }
  sub process_all_runlog {
    my ($self) = @_;
    my $files_to_process = $self->retrieve_runlogfile_urls();
    if (!$files_to_process) {
      croak 'No files to process at this time!';
    }
    $self->retrieve_runlog_xml();
    #$self->websend_runlog_xml();
    #if sending xml too large, time out before response object back
    $self->process_response_objects();
    return;
  }

  #for processing bustard files
  sub process_all_bustard {
    my ($self) = @_;

    carp 'about to process the runs';

    my $files_to_process = $self->retrieve_bustard_file_urls();

    if (!$files_to_process) {
      croak 'No files to process at this time!';
    }

    return;
  }

  sub retrieve_bustard_file_urls {

    my ($self) = @_;

    my $util = $self->util();

    $self->bustard_runs_done();
    $self->look_for_runs();
    $self->set_runs_finished({});
    if (!$self->get_response_objects()) {
      $self->set_response_objects([]);
    }

    my $has_files_to_process = 0;

    my @runs = reverse sort { $a->id_run() <=> $b->id_run() } @{$self->get_runs()};
    foreach my $run (@runs) {

      my $id_run      = $run->id_run();
      my $id_run_pair = $run->id_run_pair() || $id_run;

      $self->set_bustard_output([]);
      $self->set_bustard_file_urls([]);
#foreach (sort keys %{$self->get_runs_finished()}) { if ($_) { warn $_; } };
      if($self->check_id_run($id_run, $id_run_pair)){
        next;
      }

      carp "Processing run : $id_run\n";

      my @files;
      $has_files_to_process++;
      my $srpipe_util = srpipe::util->new();
      my $file_dir    = $self->get_file_dir();
      $file_dir       =~ s/run_no/$id_run/gxms;
      $file_dir       .= $self->get_file_ext();
      @files          = $self->find_file_urls($file_dir);

      my $prelim_analysis = $self->check_prelim_analysis(\@files);

      if ($prelim_analysis) {
        $has_files_to_process--;
	carp "nothing for $id_run";
	next;
      }

      my $score_dir = $self->get_score_dir();
      $score_dir =~ s/run_no/$id_run/gxms;
      $score_dir .= $self->get_file_ext();
      $self->find_file_urls($score_dir);

      my $summary_dir = $self->get_summary_dir();
      $summary_dir    =~ s/run_no/$id_run/gxms;
      $self->find_file_urls($summary_dir);

      my ($firecrest_dir, $bustard_dir) = $self->get_firecrest_bustard_dir($id_run);

      $self->find_signal_mean_files($id_run, $bustard_dir);

      $self->find_fastqcheck_files($id_run, $firecrest_dir);

      $self->find_matrix_files($id_run, $firecrest_dir);

      $self->find_matrix_files13($id_run, $bustard_dir);

      $self->find_offset_files($id_run, $firecrest_dir);

      undef $srpipe_util;

      if (!$self->get_bustard_file_urls()->[0]) {
        $has_files_to_process--;
        carp "nothing for $id_run";
        next;
      }
#warn $self->get_bustard_file_urls()->[0];
      my ($file_for_run) = $self->get_bustard_file_urls()->[0] =~ /IL\d+_0*(\d+)/xms ;
#warn $file_for_run;
      if ($file_for_run != $id_run && $file_for_run != $id_run_pair) {
        $has_files_to_process--;
	     carp "nothing for $id_run";
	     next;
      }

      $self->retrieve_bustard_xml();
      if (!$self->get_bustard_output() || scalar@{$self->get_bustard_output()} == 0) {
        $has_files_to_process--;
        carp "nothing for $id_run";
        next;
      }

      $self->collate_qcal_xml();
      $self->load_bustard_xml_directly();
#      $self->websend_bustard_xml();
      $self->process_response_objects();

      my $runs_finished = $self->get_runs_finished();
      $runs_finished->{$id_run_pair}++;

      if ($id_run > $id_run_pair) {
        my $temp = $id_run;
        $id_run = $id_run_pair;
        $id_run_pair = $temp;
      }

      $self->run_caching({id_run => $id_run, id_run_pair => $id_run_pair});
    }

    return $has_files_to_process;
  }

  sub find_signal_mean_files{
      my ($self, $id_run, $bustard_dir) = @_;

      my $signal_mean_dir = $self->get_signal_mean_dir();
      if($signal_mean_dir){
        $signal_mean_dir    =~ s/run_no/$id_run/gxms;
        $self->find_file_urls($signal_mean_dir);
      }elsif($bustard_dir){
        $signal_mean_dir = qq{$bustard_dir/Signal_Means.txt};
        $self->find_file_urls_with_root($signal_mean_dir);
      }
      return 1;
  }

  sub find_fastqcheck_files{
      my ($self, $id_run, $firecrest_dir) = @_;

      my $fastqcheck_dir = $self->get_fastqcheck_dir();
      if($fastqcheck_dir){
        $fastqcheck_dir   =~ s/run_no/$id_run/gxms;
        $self->find_file_urls($fastqcheck_dir);
      }elsif($firecrest_dir){
        $fastqcheck_dir = qq{$firecrest_dir/archive/*fastqcheck};
        $self->find_file_urls_with_root($fastqcheck_dir);
      }
      return 1;
  }

  sub find_offset_files{
    my ($self, $id_run, $firecrest_dir) = @_;
    my $offset_dir = $self->get_offset_dir();

    if($offset_dir){
        $offset_dir =~ s/run_no/$id_run/gxms;
        $self->find_file_urls($offset_dir);
    }elsif($firecrest_dir){
        $offset_dir = qq{$firecrest_dir/Offsets/s_*_offsets.txt};
        $self->find_file_urls_with_root($offset_dir);
    }
    return 1;
  }

  sub find_matrix_files{
    my ($self, $id_run, $firecrest_dir) = @_;

    my $matrix_dir = $self->get_matrix_dir();
    if($matrix_dir){
        $matrix_dir =~ s/run_no/$id_run/gxms;
        $self->find_file_urls($matrix_dir);
    }elsif($firecrest_dir){
        $matrix_dir = qq{$firecrest_dir/Matrix/s_*_*_matrix.txt};
        $self->find_file_urls_with_root($matrix_dir);
    }
    return 1;
  }

  sub find_matrix_files13{
    my ($self, $id_run, $bustard_dir) = @_;

    my $matrix_dir13 = $self->get_matrix_dir13();
    if($matrix_dir13){
        $matrix_dir13 =~ s/run_no/$id_run/gxms;
        $self->find_file_urls($matrix_dir13);
    }elsif($bustard_dir){
        $matrix_dir13 = qq{$bustard_dir/Matrix/s_*_*_matrix.txt};
        $self->find_file_urls_with_root($matrix_dir13);
    }
    return 1;
  }

  sub check_id_run{
    my ($self, $id_run, $id_run_pair)= @_;
 
    if ( $RUNS_TO_SKIP{$id_run} || $id_run < $MINIMUM_ID_RUN || $id_run_pair < $MINIMUM_ID_RUN || $self->get_bustard_runs_done()->{$id_run} || $self->get_bustard_runs_done()->{$id_run_pair} || $self->get_runs_finished()->{$id_run} || $self->get_runs_finished()->{$id_run_pair}) {

        return 1;
    }

    return 0;
  }

  sub check_prelim_analysis{
    my ($self, $files) = @_;
    my $prelim_analysis;
    foreach my $file (@{$files}) {
        if ($file =~ /config/xms) {
          my $text = slurp($file);
          if ($text =~ /This\ is\ a\ every-20th-tile\ run/xms) {
            $prelim_analysis++;
          }
          last;
        }
    }
    return $prelim_analysis;
  }
  sub load_bustard_xml_directly {
    my ($self) = @_;
    my $util = $self->util();

    carp 'Starting to load directly into database.';

    foreach my $xml (@{$self->get_bustard_output()}) {
      my ($dataset) = $xml =~ m/\<dataset\>(.*?)\<\/dataset\>/xms;
      carp "Loading - $dataset";
      my $ref = 'npg_qc::api::loader::'.$dataset;
      my $loader = $ref->new({util => $util, xml => $xml});
      $loader->read_xml();
    }

    return 1;
  }

  sub run_caching{
      my ($self, $arg_refs) = @_;

      my $id_run = $arg_refs->{id_run};
      my $not_double_ended;
      if ($id_run == $arg_refs->{id_run_pair}) {
        $not_double_ended++;
      }

      my $util = $self->util();
      my $api_run_caching;

      if ($self->get_run_cache_object) {
        $api_run_caching = $self->get_run_cache_object()
      } else {
        $api_run_caching = npg_qc::api::run_caching->new({util => $util});
      }
      
      print " caching lane summary data and run graph\n" or croak "can't print\n";
      eval{
        $api_run_caching->cache_query_lane_summary($id_run, 1, 1);
        $api_run_caching->run_graph($id_run, 1, 1);
        $api_run_caching->instrument_statistics($id_run, 1);
        if (!$not_double_ended) {
          $api_run_caching->cache_query_lane_summary($id_run, 2, 1);
          $api_run_caching->run_graph($id_run, 2, 1);
          $api_run_caching->instrument_statistics($id_run, 2);
        }
        1;
      } or do{
        croak $EVAL_ERROR;
      };

      return;
  }
  sub find_file_urls {
    my ($self, $file_dir) = @_;

    my $srpipe_util;
    if (!($srpipe_util = $self->get_srpipe_util())) {
      $srpipe_util = srpipe::util->new();
      $self->set_srpipe_util($srpipe_util);
    }

    my @files = $srpipe_util->my_glob($self->get_file_root(), $file_dir);

    if ($files[0] && $files[0] !~ /tmp/xms) {
      push @{$self->get_bustard_file_urls()}, @files;
    }

    return @files;
  }
  sub find_file_urls_with_root {
    my ($self, $file_dir) = @_;

    my $srpipe_util;
    if (!($srpipe_util = $self->get_srpipe_util())) {
      $srpipe_util = srpipe::util->new();
      $self->set_srpipe_util($srpipe_util);
    }

    my @files = $srpipe_util->my_glob($file_dir);

    if ($files[0] && $files[0] !~ /tmp/xms) {
      push @{$self->get_bustard_file_urls()}, @files;
    }

    return @files;
  }
  sub collate_qcal_xml {
    my ($self) = @_;
    my $qcals;
    my $parser = $self->util->parser();
    my $xmls;
    
    foreach my $xml (@{$self->get_bustard_output()}) {
      if ($xml !~ /<dataset>qcal<\/dataset/xms) {
        push @{$xmls}, $xml;
        next;
      }
      push @{$qcals}, $xml;
    }

    if (!$qcals || scalar@{$qcals} == 0) {
      return 1;
    }

    my @rows;
    foreach my $xml (@{$qcals}) {
      my $parsed_xml = $parser->parse_string($xml);
      my $row = $parsed_xml->getElementsByTagName('row')->[0];
      push @rows, $row;
    }

    my $xml = qq{<?xml version="1.0" encoding="utf-8"?><root>\n<dataset>qcal</dataset>\n<rows>\n};
    foreach my $row (@rows) {

      #############
      # ensures no null values sent
      my $twenty = $row->getAttribute('twenty');
      $twenty ||= 0;
      $row->setAttribute('twenty', $twenty);
      my $twentyfive = $row->getAttribute('twentyfive');
      $twentyfive ||= 0;
      $row->setAttribute('twentyfive', $twentyfive);
      my $thirty = $row->getAttribute('thirty');
      $thirty ||= 0;
      $row->setAttribute('thirty', $thirty);

      $xml .= $row->toString();
    }
    $xml .= qq{\n</rows>\n</root>};

    push @{$xmls}, $xml;
    $self->set_bustard_output($xmls);

    return 1;
  }

  sub retrieve_bustard_xml {
    my ($self) = @_;
    my $util = $self->util();

    foreach my $file_url (@{$self->get_bustard_file_urls()}) {
      my $bustard_output = npg_qc::api::process::bustard_output->new({util => $util});

      if (!$self->get_bustard_output()) {
        $self->set_bustard_output([]);
      }
      my $xml = $bustard_output->convert_to_xml($file_url);

      if ($xml) {

        if ($xml =~ m/\<dataset\>summary_data\<\/dataset\>/xms) {
          push @{$self->get_bustard_output()}, $bustard_output->reprocess_summary($xml);
        } else {
          push @{$self->get_bustard_output()}, $xml;
	}
      }
    }

    return;
  }

  sub websend_bustard_xml {
    my ($self) = @_;
    my $util = $self->util();

    my $return_value;

    my $number_to_send = scalar@{$self->get_bustard_output()};
    carp qq{$number_to_send xml files to be sent to webserver.};

    if ($number_to_send < $MINIMUM_NUMBER_OF_EXPECTED_TILE_SCORE_XML_STRINGS) {
      carp qq{\n\nWARNING: VERY LOW NUMBER OF XML STRINGS - ONLY $number_to_send\n\n};
    }

    foreach my $xml (@{$self->get_bustard_output()}) {
      $return_value = $self->websend_any_xml($xml);
    }

    ##########
    # return the last value returned by websend_any_xml, in case it is useful
    return $return_value;
  }

  sub process_response_objects {
    my ($self) = @_;

    my $parser = $self->util->parser();
    my $sent_type_counts = {};
    if($self->get_response_objects()){
      foreach my $response (@{$self->get_response_objects()}) {
        # warn $response->content;
        my $sent_type;
        eval {
          $sent_type = $parser->parse_string($response->content)->getElementsByTagName('sent_type');
          $sent_type_counts->{$sent_type}++;
          1;
        } or do {
          carp $EVAL_ERROR;
          carp 'error in xml response object';
        };
      }
    }

    $self->set_response_objects(undef);

    return $sent_type_counts;
  }

  sub look_for_runs {
    my ($self) = @_;
    carp 'looking for runs';
    my $util  = $self->util();
    my $npg_api_util = $util->get_npg_api_util();
    $npg_api_util->base_uri($self->npg_uri());
    my $tries = $TRIES;    # number of run_status_dict->new retries
    my ($runs, $runs_ac, $runs_ap, $runs_aip, $runs_ra, $runs_qcc, $runs_qcrp);

    while ($tries-- > 0) {        # retry to get around network timeouts

      eval {

        my $npg_rsd_obj_ac = npg::api::run_status_dict->new({
          description => 'analysis complete',
        });
	$npg_rsd_obj_ac->{util} = $npg_api_util;
        my $npg_rsd_obj_qcrp = npg::api::run_status_dict->new({
          description => 'qc review pending',
        });
	$npg_rsd_obj_qcrp->{util} = $npg_api_util;
        my $npg_rsd_obj_qcc = npg::api::run_status_dict->new({
          description => 'qc complete',
        });
	$npg_rsd_obj_qcc->{util} = $npg_api_util;
        my $npg_rsd_obj_ap = npg::api::run_status_dict->new({
          description => 'archival pending',
        });
	$npg_rsd_obj_ap->{util} = $npg_api_util;
        my $npg_rsd_obj_aip = npg::api::run_status_dict->new({
          description => 'archival in progress',
        });
	$npg_rsd_obj_aip->{util} = $npg_api_util;
        my $npg_rsd_obj_ra = npg::api::run_status_dict->new({
          description => 'run archived',
        });
	$npg_rsd_obj_ra->{util} = $npg_api_util;

        if (!$runs_ac)   { $runs_ac   = $npg_rsd_obj_ac->runs();    }
        if (!$runs_ap)   { $runs_ap   = $npg_rsd_obj_ap->runs();    }
        if (!$runs_aip)  { $runs_aip  = $npg_rsd_obj_aip->runs();   }
        if (!$runs_ra)   { $runs_ra   = $npg_rsd_obj_ra->runs();    }
        if (!$runs_qcc)  { $runs_qcc  = $npg_rsd_obj_qcc->runs();   }
        if (!$runs_qcrp) { $runs_qcrp = $npg_rsd_obj_qcrp->runs();  }

        $tries = $TRY_SUCCESS;
        1;

      } or do {

        carp $EVAL_ERROR;
        carp 'failed to obtain runs -- retrying';

      };

    }

    if ($tries == 0) {
      croak 'too many retries -- quitting';
    }

    $runs = [@{$runs_ac},@{$runs_ap},@{$runs_aip},@{$runs_ra},@{$runs_qcc},@{$runs_qcrp}];

    $self->set_runs($runs);
    return;
  }
  
  sub bustard_runs_done {
    my ($self) = @_;
    my $util = $self->util();

    carp 'looking for runs done';

    my $summary_obj = npg_qc::api::summary->new({util => $util});
    my $summaries = $summary_obj->runs_with_fully_complete_pairs();

    my %run_ids;
    foreach my $summary (@{$summaries}) {
      $run_ids{$summary->id_run()}++;
    }

    $self->set_bustard_runs_done(\%run_ids);
    return;
  }
  
  sub websend_any_xml {
    my ($self, $xml) = @_;
    my $util = $self->util();

    if ($xml !~ m/\<dataset\>run_config\<\/dataset\>/xms) {
      my $parsed_xml = $util->parser->parse_string($xml);
    }

    if (!$self->get_response_objects()) {
      $self->set_response_objects([]);
    }

    my $response = $util->post(
      $util->base_uri().q{create_xml},
      [
        'Content_Type'   => 'application/xml',
  			'Content_Length' => length $xml,
  			'Content'        => $xml,
  			'Accept'         => 'text/xml',
			]
    );

    if ($response->is_success()) {
      push @{$self->get_response_objects()}, $response;
    } else {
      carp 'failed to successfully post - this could be from a timeout issue';
      return 0;
    }

    return 1;
  }

  sub get_run_folder_from_summary_link{
    my ($self, $root_dir, $file_dir) = @_;

    my $srpipe_util;
    if (!($srpipe_util = $self->get_srpipe_util())) {
      $srpipe_util = srpipe::util->new();
      $self->set_srpipe_util($srpipe_util);
    }

    my @files = $srpipe_util ->my_glob($root_dir, $file_dir);
    my $run_folder = $files[0];

    if($run_folder){

      my $gerald_dir = readlink "$run_folder/Latest_Summary";
      if($gerald_dir){

        my @dirs = split /\//mxs, $gerald_dir;
        my $firecrest_dir = qq{$run_folder/Data/$dirs[1]};
        my $bustard_dir = qq{$firecrest_dir/$dirs[2]};
        $gerald_dir = qq{$run_folder/$gerald_dir};
        #print "$run_folder\n$firecrest_dir\n$bustard_dir\n$gerald_dir\n";
        return [$run_folder, $firecrest_dir, $bustard_dir, $gerald_dir];
      }
    }
    return [];
  }

  sub get_firecrest_bustard_dir{
    my ($self, $id_run) = @_;

    my $run_folder = $self->get_run_folder();
    my $firecrest_dir;
    my $bustard_dir;
    if($run_folder){
        $run_folder =~ s/run_no/$id_run/gxms;
        my $run_folders
          = $self->get_run_folder_from_summary_link($self->get_file_root(), $run_folder);
        $firecrest_dir = $run_folders->[1];
        $bustard_dir = $run_folders->[2];
    }
    return ($firecrest_dir, $bustard_dir);
   }
}
1;
__END__

=head1 NAME

npg_qc::api::main

=head1 VERSION

$LastChangedRevision: 4573 $

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 new - Constructor

  my $oDerived = npg_qc::api::main->new({'util' => $oUtil,});

  my $oDerived = npg_qc::api::main->new({
    # 'key' => value parameters for the derived class
  });

=head2 process_all_bustard - handler to run and it does all in correct order for you, with no need to go through each method in turn in script

=head2 collate_qcal_xml - handler collates all the qcal information into one xml for sending to the web, as each only has one row ordinarily

=head2 retrieve_bustard_file_urls - finds all the relevant Bustard file uris and stores them

=head2 look_for_runs - stores an array of npg::api::run objects for each run which has a status post analysis

=head2 bustard_runs_done - stores a hash where keys are the id_runs for each of the runs already processed

=head2 retrieve_bustard_xml - retrieves bustard files, and processes them via process::bustard_output into xml

=head2 websend_bustard_xml - sends all the bustard xml strings to npg-qc via LWP::UserAgent

=head2 websend_any_xml - sends a single xml string to npg-qc via LWP::UserAgent

=head2 process_response_objects - processes the response objects from websend_xml

=head2 find_file_urls - refactor of code to reduce complexity of retrieve_bustard_file_urls

=head2       process_all_runlog  - send each runlog data to the database by web services

=head2       retrieve_runlog_xml - for one given runlog file, get the list xml strings and send to the web server

=head2       retrieve_runlogfile_urls - get a list of runlog files to be processed

=head2       runs_done_runlog  - a list of id_run already in the database with runlog data

=head2       websend_runlog_xml - send a list of xml strings to the web server

=head2       find_file_urls_with_root

=head2       get_run_folder_from_summary_link

=head2       get_firecrest_bustard_dir

=head2       check_id_run

=head2       find_offset_files 

=head2       find_matrix_files

=head2       find_matrix_files13

=head2       find_fastqcheck_files

=head2       find_signal_mean_files

=head2 util - util instance for this resource set

  Returns a cached object if one exists, or creates and caches a new one

  my $oUtil = $oDerived->util();

=head2 new_from_xml - create and populate an npg::api::* from a XML::LibXML::Element

  my $oObj = $oDerived->new_from_xml('npg::api::<package>', $oXMLElement);

=head2 hasa - makes one:one accessors on compilation

 package npg::api::<base-subclass>;
 use strict;
 use warnings;

 __PACKAGE__->hasa(['child1', 'child2', {'current_child3' => 'child3-package'}]);

=head2 hasmany - makes one:many accessors on compilation

 package npg::api::<base-subclass>;
 use strict;
 use warnings;

 __PACKAGE__->hasmany(['child1', 'child2', {'current_child3' => 'child3-package'}]);

=head2 get - overridden get accessor from Class::Accessor

Fetches XML over HTTP based on self->util->base_uri().

=head2 list - default handling for reading lists of objects

  my $oDOM = $oDerived->list();

=head2 read - default handling for reading individual objects

  my $oDOM = $oDerived->read();

=head2 create - default handling object creation

  my $oDOM = $oDerived->create();

  This creates an HTTP POST request with a multipart/form-data flavour
  and pushes object member data, for a new entity, up to the
  create_xml aspects of the service.

=head2 update - default handling object update

  my $oDOM = $oDerived->update();

  This creates an HTTP POST request with a multipart/form-data flavour
  and pushes object member data, for an existing entity ,up to the
  update_xml aspects of the service.

=head2 retry - method retries connecting to npg_qc before croaking with an error=head1 DIAGNOSTICS

=head2 run_caching - run all caching 

=head2 load_bustard_xml_directly

=head2 check_prelim_analysis

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

strict
warnings
Class::Std
Carp
English
npg_qc::api::base
Perl6::Slurp
npg_qc::api::process::swift_output
npg_qc::api::process::bustard_output
npg::api::run_status_dict
srpipe::util

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Andy Brown, E<lt>ajb@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2008 GRL, by Andy Brown

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.4 or,
at your option, any later version of Perl 5 you may have available.

=cut
