#########
# Author:        ajb
# Maintainer:    $Author: gq1 $
# Created:       2008-07-22
# Last Modified: $Date: 2009-01-21 16:02:20 +0000 (Wed, 21 Jan 2009) $
# Id:            $Id: run_config.pm 4034 2009-01-21 16:02:20Z gq1 $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/trunk/lib/npg_qc/api/run_config.pm $
#
package npg_qc::api::run_config;
use strict;
use warnings;
use Class::Std;
use Carp;
use base qw(npg_qc::api::base);
use Readonly;

our $VERSION = do { my ($r) = q$LastChangedRevision: 4034 $ =~ /(\d+)/mxs; $r; };

{
  ## no critic (ProhibitUnusedVariables)
  my %run_configs_of   :ATTR( :get<run_configs>,   :set<run_configs>   );
  my %id_run_config_of :ATTR( :get<id_run_config>, :set<id_run_config> );
  my %id_run_of        :ATTR( :get<id_run>,        :set<id_run>        );
  my %config_text_of   :ATTR( :get<config_text>,   :set<config_text>   );
  ## use critic

  sub fields {
    return qw(
              id_run_config
              id_run
              config_text
            );
  }

  sub id_run_config {
    my ($self, $id_run_config) = @_;
    if ($id_run_config) {
      $self->set_id_run_config($id_run_config);
    }
    return $self->get_id_run_config();
  }

  sub id_run {
    my ($self, $id_run) = @_;
    if ($id_run) {
      $self->set_id_run($id_run);
    }
    return $self->get_id_run();
  }

  sub config_text {
    my ($self, $config_text) = @_;
    if ($config_text) {
      $self->set_config_text($config_text);
    }
    return $self->get_config_text();
  }

  sub run_configs {
    my ($self) = @_;
    my $util = $self->util();
    my $pkg        = ref $self;
    my ($obj_type) = $pkg =~ /([^:]+)$/mxs;
    my $obj_pk     = $self->primary_key();
    my $obj_uri    = sprintf '%s/%s', $util->base_uri(), $obj_type;
    my $response   = $util->get($obj_uri);

    if(!$response->is_success()) {
      croak $response->status_line();
    }

    $self->set_run_configs($util->parser->parse_string($response->content()));
    my $run_configs = $self->get_run_configs()->getElementsByTagName('run_configs')->[0];

    return [map { $self->new_from_xml($pkg, $_) } $run_configs->getElementsByTagName('run_config')];
  }
}
1;
__END__

=head1 NAME

npg_qc::api::run_config

=head1 VERSION

$LastChangedRevision: 4034 $

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 new - Constructor

  my $oRunConfig = npg_qc::api::run_config->new({'util' => $oUtil,});

=head2 fields - returns an array of the fields in the run_config table

  my $aFields = $oRunConfig->fields();

=head2 run_configs - fetches all run configs from the npg_qc application, and turns into array of npg_qc::api::run_config objects

  my $aRunConfigs = $oRunConfig->run_configs();

=head2 id_run_config - accessor for id_run_config. takes an id as an optional parameter (which it stores) and returns the id_run_config

  my $id_run_config = $oRunConfig->id_run_config($id_run_config);

=head2 id_run - accessor for id_run. takes an id as an optional parameter (which it stores) and returns the id_run

  my $id_run = $oRunConfig->id_run($id_run);

=head2 config_text - accessor for config_text. takes a config_text as an optional parameter (which it stores) and returns the text

  my $config_text = $oRunConfig->config_text($config_text);

=head2 util - util instance for this resource set

  Returns a cached object if one exists, or creates and caches a new one

  my $oUtil = $oDerived->util();

=head2 new_from_xml - create and populate an npg::api::* from a XML::LibXML::Element

  my $oObj = $oDerived->new_from_xml('npg::api::<package>', $oXMLElement);

=head2 get - overridden get accessor from Class::Accessor

Fetches XML over HTTP based on self->util->base_uri().

=head2 list - default handling for reading lists of objects

  my $oDOM = $oDerived->list();

=head2 read - default handling for reading individual objects

  my $oDOM = $oDerived->read();

=head2 create - default handling object creation

  my $oDOM = $oDerived->create();

  This creates an HTTP POST request with a multipart/form-data flavour
  and pushes object member data, for a new entity, up to the
  create_xml aspects of the service.

=head2 update - default handling object update

  my $oDOM = $oDerived->update();

  This creates an HTTP POST request with a multipart/form-data flavour
  and pushes object member data, for an existing entity ,up to the
  update_xml aspects of the service.

=head2 retry - method retries connecting to npg_qc before croaking with an error=head1 DIAGNOSTICS

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

strict
warnings
Class::Std
Carp
npg_qc::api::base
Readonly

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Andy Brown, E<lt>ajb@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2008 GRL, by Andy Brown

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.4 or,
at your option, any later version of Perl 5 you may have available.

=cut
