#########
# Author:        gq1
# Maintainer:    $Author: gq1 $
# Created:       2009-2-27
# Last Modified: $Date: 2009-03-10 10:38:36 +0000 (Tue, 10 Mar 2009) $
# Id:            $Id: run_recipe.pm 4437 2009-03-10 10:38:36Z gq1 $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/trunk/lib/npg_qc/api/run_recipe.pm $
#
package npg_qc::api::run_recipe;
use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use base qw(npg_qc::api::base);
use Readonly;
use srpipe::identify::missing::tiles;
use srpipe::util;
use npg::api::run;


our $VERSION = do { my ($r) = q$LastChangedRevision: 4437 $ =~ /(\d+)/mxs; $r; };

Readonly our $COLS_TILE_NUM =>{330 =>3,
										 300 =>3,
										 50 =>1,
                              };
{

  ##no critic (ProhibitUnusedVariables)
  my %id_run_recipe_of  :ATTR( :get<id_run_recipe>,   :set<id_run_recipe>  );
  my %id_run_of         :ATTR( :get<id_run>,          :set<id_run>         );
  my %cycle_of          :ATTR( :get<cycle>,           :set<cycle>          );
  my %lane_of           :ATTR( :get<lane>,            :set<lane>           );
  my %tile_of           :ATTR( :get<tile>,            :set<tile>           );
  my %col_of            :ATTR( :get<col>,             :set<col>            );
  my %dbh_of            :ATTR( :get<dbh>,             :set<dbh>            );

  my %file_root_of      :ATTR( :get<file_root>,       :set<file_root>      );
  my %file_outgoing_of  :ATTR( :get<file_outgoing>,   :set<file_outgoing>  );
  my %file_analysis_of  :ATTR( :get<file_analysis>,   :set<file_analysis>  );
  my %runlist_done_of   :ATTR( :get<runlist_done>,    :set<runlist_done>   );
  my %runfolder_list_of :ATTR( :get<runfolder_list>,  :set<runfolder_list> );
  my %runs_runtile_of   :ATTR( :get<runlist_runtile>, :set<runlist_runtile>);
  ##use critic
 
  sub fields {
    return qw(
              id_run_recipe
              id_run
              cycle
              lane
              tile
              col
            );
  }

  sub id_run {
    my ($self, $id_run) = @_;
    if ($id_run) {
      $self->set_id_run($id_run);
    }
    return $self->get_id_run();
  }

  sub init{
    my ($self) = @_;

    my $ref   = [];
    if($self->get_id_run() && !$self->get_id_run_recipe()){

      my $query = q(SELECT id_run_recipe
                  FROM   run_recipe
                  WHERE  id_run = ?
                 );

      my $dbh = $self->dbh();

      eval {
        $ref = $dbh->selectall_arrayref($query, {}, $self->get_id_run());

      1;
      } or do {
        carp $EVAL_ERROR;
        return;
      };
    }

    if(@{$ref}) {
      $self->set_id_run_recipe($ref->[0]->[0]);
    }

    return 1;
  }

  sub dbh {
    my ($self, $dbh) = @_;
    if ($dbh) {
      $self->set_dbh($dbh);
    }
    if (!$self->get_dbh()) {
      $dbh = $self->util->dbh();
      $self->set_dbh($dbh);
    }
    return $self->get_dbh();
  }

  sub save_data {
    my ($self) = @_;
    
    my $query = q{INSERT INTO run_recipe (id_run, cycle, lane, tile, col) VALUES (?,?,?,?,?)};

    my $dbh = $self->dbh();
    eval {
      $dbh->do($query,
               {},
               $self->get_id_run(),
               $self->get_cycle(),
               $self->get_lane(),
               $self->get_tile(),
               $self->get_col()
              );
      $dbh->commit();
      1;
    } or do {
      if ($EVAL_ERROR =~ /Duplicate\ entry/xms) {
        carp "$EVAL_ERROR - continuing anyway";
      } else {
        $dbh->rollback();
        croak $EVAL_ERROR;
      }
    };

    return 1;
  }
  
  sub update {
    my ($self) = @_;
    
    my $query = q{update run_recipe set cycle = ?, lane = ?, tile = ?, col = ? where id_run_recipe = ?};

    my $dbh = $self->dbh();
    eval {
    
      $dbh->do($query,
               {},
               $self->get_cycle(),
               $self->get_lane(),
               $self->get_tile(),
               $self->get_col(),
               $self->get_id_run_recipe()
              );
      $dbh->commit();
      1;
    } or do {
        $dbh->rollback();
        croak $EVAL_ERROR;
    };

    return 1;
  }

  sub read_run_recipe {
    my ($self, $run_folder) = @_;

    my $identify_missing_tiles = srpipe::identify::missing::tiles->new({});

    $identify_missing_tiles->parse_recipe_file($run_folder);
    
    my $col_count = $identify_missing_tiles->get_col_count();
    my $cycle_count = $identify_missing_tiles->get_cycle_count();
    my $tile_count = $identify_missing_tiles->get_tile_count();
    my $lane_count = $identify_missing_tiles->get_lane_count();

    $self->set_col($col_count);
    $self->set_cycle($cycle_count);
    $self->set_tile($tile_count);
    $self->set_lane($lane_count);

    return 1;
  }
  
  sub get_runlist_done_from_db{
    my ($self) = @_;
    print "getting a list of runs alreay in the database....\n" or croak "can't print\n";
    $self->set_runlist_done({});

    my $query = q{SELECT id_run FROM run_recipe};

    my $dbh = $self->dbh();
    my $ref;
    eval {
      $ref = $dbh->selectall_arrayref($query, {});
      my $num_runs_in_db = scalar @{$ref};
      if($num_runs_in_db) {
        foreach my $row (@{$ref}){
           $self->get_runlist_done()->{$row->[0]} = 1;
        }
      }
      print "There are $num_runs_in_db in the database with run recipe data\n" or croak "can't print\n";
      1;
    } or do {
        croak $EVAL_ERROR;
    };
    return 1;
  }

  sub retrieve_runfolder_list {
    my ($self) = @_;

    $self->get_runlist_done_from_db();
    
    $self->set_runfolder_list([]);

    my $srpipe_util = srpipe::util->new();
    #$self->set_file_root('t/data');   
    #my $file_outgoing = 'staging/IL*/outgoing/*/';
    #my $file_analysis = 'staging/IL*/analysis/*/';
    
    my @runfolder_list_outgoing
      = $srpipe_util->my_glob($self->get_file_root(), $self->get_file_outgoing());
    
    my @runfolder_list_analysis
      = $srpipe_util->my_glob($self->get_file_root(), $self->get_file_analysis());
         
	 foreach my $runfolder (@runfolder_list_outgoing, @runfolder_list_analysis){

      my @dirs = split /Recipe/ms, $runfolder;
      $runfolder = $dirs[0];
      my ($id_run) = $runfolder =~ /_0*(\d+)/xms ;
      print ">Getting id_run from runfolder $runfolder: $id_run\n" or croak "can't print\n";

      if ($self->get_runlist_done()->{$id_run}){
          print "  already done\n" or croak "can't print\n";
          next;
      }

      push @{$self->get_runfolder_list()}, $runfolder;
    }
    my $has_files_to_process = scalar @{$self->get_runfolder_list()};
    print "$has_files_to_process runfolders need to be processed\n" or croak "can't print\n";
    return $has_files_to_process;
  }

  sub process_all_runfolder{
    my ($self) = @_;

    my $util = $self->util();
    my $runfolder_todo = $self->retrieve_runfolder_list();

    if($runfolder_todo == 0 ){
      print "There is no any runfolder to do currently\n" or croak "can't print\n";
    }else{
        
      foreach my $runfolder (@{$self->get_runfolder_list()}){
        my ($id_run) = $runfolder =~ /_0*(\d+)/xms ;
        print ">Processing runfolder $runfolder\n" or croak "can't print\n";
        
        my $run_recipe_obj = npg_qc::api::run_recipe->new({util => $util });
        $run_recipe_obj->id_run($id_run);
        eval{
          $run_recipe_obj->read_run_recipe($runfolder);
          1;
        } or do {
          carp $EVAL_ERROR;
        };

        if($run_recipe_obj->get_cycle()
            && $run_recipe_obj->get_col()
            && $run_recipe_obj->get_lane()
            && $run_recipe_obj->get_tile()
            && $run_recipe_obj->id_run()
        ){
           $run_recipe_obj->save_data();
        } else{
           print "  no data stored for run $id_run\n" or croak "can't print\n";
        }
      }
    }
    return 1;
  }

  sub paired_id_run{
    my ($self, $id_run) = @_;

    my $util  = $self->util();

    my $npg_api_util = $util->get_npg_api_util();
    print "  trying to get the paired id run for run $id_run from NPG\n" or croak "can't print\n";
    $npg_api_util->base_uri($self->npg_uri());
    my $npg_run = npg::api::run->new({
          id_run => $id_run,
          util   =>$npg_api_util,
    });
    return $npg_run->id_run_pair();
  }

  sub paired_id_run_from_db{
    my ($self, $id_run) = @_;
    
    my $paired_id_run = 0;
    print "  trying to get the paired id run for run $id_run directly from DB\n" or croak "can't print\n";
    my $query = q(SELECT id_run, id_run_pair
                  FROM run_and_pair
                  WHERE id_run = ? or id_run_pair = ?
                 );
    eval{
      my $dbh = $self->dbh();
      my $sth = $dbh->prepare($query);
      $sth->execute($id_run, $id_run);

      my @row = $sth->fetchrow_array();
      if(scalar @row ){
        $paired_id_run =($row[0] == $id_run)? $row[1] : $row[0];
      }
      1;
    } or do {
        croak $EVAL_ERROR;
    };
    return $paired_id_run;
  }
  sub get_cycle_from_db{
    my ($self, $id_run) = @_;

    my $query = q(SELECT cycle
                  FROM   run_graph
                  WHERE  id_run = ?
                 );

    my $dbh = $self->dbh();
    my $cycle;

    eval {
      my $ref = $dbh->selectall_arrayref($query, {}, $id_run);
      if(scalar @{$ref}){
        $cycle = $ref->[0]->[0];
      }
      1;
    } or do {
        croak $EVAL_ERROR;
    };
    return $cycle;
  }

  sub get_runlist_from_runtile{
    my ($self) = @_;
    print "getting a list of runs from run_tile table....\n"
    or croak "can't print\n";

    my $query = q{SELECT rt.id_run, COUNT(DISTINCT rt.tile) AS tile_count,
                  COUNT(DISTINCT rt.position) AS lane_count
                  FROM run_tile rt
                  WHERE rt.end  = 1
                  GROUP BY rt.id_run
                 };

    my $runlist = [];

    eval {

      my $dbh = $self->dbh();
      my $sth = $dbh->prepare($query);
      $sth->execute();

      while(my @row = $sth->fetchrow_array()){
 
        push @{$runlist}, \@row;
      }

    
      my $num_runs_from_runtile = scalar @{$runlist};

      print "There are $num_runs_from_runtile runs in the database\n" or croak "can't print\n";
      1;
    } or do {
        croak $EVAL_ERROR;
    };
    
    $self->set_runlist_runtile($runlist);
 
    return $runlist;
  }

  sub save_one_run_data{
    my ($self, $id_run, $tile, $lane, $cycle) = @_;
    
    my $util = $self->util();

    my $col = $COLS_TILE_NUM->{$tile} ? $COLS_TILE_NUM->{$tile} : 2;

    my $run_recipe_obj = npg_qc::api::run_recipe->new({util => $util });
    $run_recipe_obj->id_run($id_run);
    $run_recipe_obj->set_tile($tile);
    $run_recipe_obj->set_col($col);
    $run_recipe_obj->set_lane($lane);
    $run_recipe_obj->set_cycle($cycle);

    if($run_recipe_obj->get_cycle()
            && $run_recipe_obj->get_col()
            && $run_recipe_obj->get_lane()
            && $run_recipe_obj->get_tile()
            && $run_recipe_obj->id_run()
    ){
        $run_recipe_obj->save_data();
    } else{
        print "  no data stored for run $id_run\n" or croak "can't print\n";
    }
    return 1;
  }

  sub process_all_runs_from_runtile{
    my ($self) = @_;

    my $util = $self->util();
    
    if(!$self->get_runlist_runtile()){
      $self->get_runlist_from_runtile();
    }
    
    if(!$self->get_runlist_done()){
      $self->get_runlist_done_from_db();
    }

    foreach my $run (@{$self->get_runlist_runtile()}){
        my $id_run = $run->[0];
        my $tile = $run->[1];
        my $lane = $run->[2];

        print ">Processing run $id_run:\n" or croak "can't print\n";
       
        my $paired_id_run = $self->paired_id_run_from_db($id_run);
        if(!$paired_id_run){
          $paired_id_run = $self->paired_id_run($id_run);
        }

        if($self->get_runlist_done()->{$id_run}
            && ( ($paired_id_run && $self->get_runlist_done()->{$paired_id_run})
                 || !$paired_id_run ) ){
            print "  the data for this run and its paired run if any already in the database\n" or croak "can't print\n";
            next;
        }

        my $cycle = $self->get_cycle_from_db($id_run);

        if(! $cycle){
          $cycle = $self->get_cycle_count_direct($id_run);
        }

        if(! $cycle){
          print "  cycle number not available for run $id_run from DB\n" or croak "can't print\n";
          next;
        }

        if(!$self->get_runlist_done()->{$id_run}){
          print "  trying to save data for run $id_run\n" or croak "can't print\n";
          $self->save_one_run_data($id_run, $tile, $lane, $cycle);
          $self->get_runlist_done()->{$id_run} = 1;
        }

        if($paired_id_run && (!$self->get_runlist_done()->{$paired_id_run}) ){
          print " >trying to save data for paired run $paired_id_run:\n" or croak "can't print\n";
          $self->save_one_run_data($paired_id_run, $tile, $lane, $cycle);
          $self->get_runlist_done()->{$paired_id_run} = 1;
        }

    }
    $self->dbh()->commit();
    return 1;
  }

sub get_cycle_count_direct{
    my ($self, $id_run) = @_;
    my $table_name = q{errors_by_cycle};
    my $cycle_count = $self->get_cycle_count_from_db($id_run, $table_name);
    
    if(!$cycle_count){
      $table_name = q{move_z};
      $cycle_count = $self->get_cycle_count_from_db($id_run, $table_name);
    }
    return $cycle_count;
}

sub get_cycle_count_from_db{
    my ($self, $id_run, $table_name) = @_;

    my $query = qq{SELECT MAX(cycle) FROM $table_name ec, run_tile rt}
             .q{ WHERE rt.id_run = ? AND rt.id_run_tile = ec.id_run_tile};
    my $cycle_count;
    eval {
      my $dbh = $self->util->dbh();
      #print $query, "\n";
      my $sth = $dbh->prepare($query);
      
      $sth->execute($id_run);

      my @row = $sth->fetchrow_array();
      if(@row){
        $cycle_count = $row[0];
      }
      1;
    } or do {
      croak $EVAL_ERROR;
    };
    if(!$cycle_count){
    }
    return $cycle_count;
}
}
1;
__END__

=head1 NAME

npg_qc::api::run_recipe

=head1 VERSION

$LastChangedRevision: 4437 $

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 update - if all fields available, including id_run_recipe, update the record

=head2 save_data - insert a new record into the database

=head2 init - if id_run known, get the id_run_recipe

=head2 id_run - another access method for id_run

=head2 dbh - access method for dbh

=head2 fields - a list of fields for run_recipe

=head2 get_runlist_done_from_db - get the list of runs already in the database

=head2 process_all_runfolder - process all runfolders not being done before

=head2 read_run_recipe - given a run folder, read the data out from recipe and tile layout xml

=head2 retrieve_runfolder_list - get a list of run folder not being done before

=head2 get_runlist_from_runtile - get a list of run with tile, lane count from run_tile table

=head2 get_cycle_from_db - given id_run, get the cached cycle_count from run_graph table

=head2 get_cycle_count_direct - given id_run, get the cycle_count directly from move_z or errors_by_cycle table

=head2 get_cycle_count_from_db - given id_run and a table name cotaining cycle information to get cycle number

=head2 paired_id_run - given id_run, get the paired run id from npg api

=head2 paired_id_run_from_db - given id_run, get the paired run id directly from npgqc database

=head2 process_all_runs_from_runtile - process the list of runs from run_tile table and store them into database if not 

=head2 save_one_run_data - given a set of data for one run, store them in the table

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

strict
warnings
Carp
English
npg_qc::api::base
Readonly;
srpipe::identify::missing::tiles;
srpipe::util;

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Guoying Qi, E<lt>gq1@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2008 GRL, by Guoying Qi

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.4 or,
at your option, any later version of Perl 5 you may have available.

=cut
