#########
# Author:        gq1
# Maintainer:    $Author: gq1 $
# Created:       2008-12-05
# Last Modified: $Date: 2009-03-10 10:38:36 +0000 (Tue, 10 Mar 2009) $
# Id:            $Id: run_timeline.pm 4437 2009-03-10 10:38:36Z gq1 $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/trunk/lib/npg_qc/api/run_timeline.pm $
#
package npg_qc::api::run_timeline;
use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use base qw(npg_qc::api::base);
use Readonly;


use npg::api::run;

our $VERSION = do { my ($r) = q$LastChangedRevision: 4437 $ =~ /(\d+)/mxs; $r; };

Readonly our $TRY_SUCCESS => -1;
Readonly our $NUM_TRIES => 10;
Readonly our $MINIMUM_ID_RUN => 2200;

{

  ##no critic (ProhibitUnusedVariables)
  my %id_run__timeline  :ATTR( :get<id_run_timeline>, :set<id_run_timeline>);
  my %id_run_of         :ATTR( :get<id_run>,          :set<id_run>         );
  my %start_time        :ATTR( :get<start_time>,      :set<start_time>     );
  my %end_time          :ATTR( :get<end_time>,        :set<end_time>       );
  my %complete_time     :ATTR( :get<complete_time>,   :set<complete_time>  );
  my %dbh               :ATTR( :get<dbh>,             :set<dbh>            );
  my %run_list_all      :ATTR( :get<run_list_all>,    :set<run_list_all>   );
  my %run_list_done     :ATTR( :get<run_list_done>,    :set<run_list_done> );
  ##use critic
 
  sub fields {
    return qw(
              id_run_timeline
              id_run
              start_time
              complete_time
              end_time
            );
  }

  sub id_run {
    my ($self, $id_run) = @_;
    if ($id_run) {
      $self->set_id_run($id_run);
    }
    return $self->get_id_run();
  }

  sub init{
    my ($self) = @_;

    my $ref   = [];
    if($self->get_id_run() && !$self->get_id_run_timeline()){

      my $query = q(SELECT id_run_timeline
                  FROM   run_timeline
                  WHERE  id_run = ?
                 );

      my $dbh = $self->dbh();

      eval {
        $ref = $dbh->selectall_arrayref($query, {}, $self->get_id_run());

      1;
      } or do {
        carp $EVAL_ERROR;
        return;
      };
    }

    if(@{$ref}) {
      $self->set_id_run_timeline($ref->[0]->[0]);
    }

    return 1;
  }
  sub get_analysis_date_from_db{
    my ($self) = @_;

    my $ref   = [];
    if($self->get_id_run() && !$self->get_id_run_timeline()){

      my $query = q(select date from analysis where id_run = ?
                 );

      my $dbh = $self->dbh();

      eval {
        $ref = $dbh->selectall_arrayref($query, {}, $self->get_id_run());

      1;
      } or do {
        carp $EVAL_ERROR;
        return;
      };
    }

    if(@{$ref}) {
      return $ref->[0]->[0];
    }

    return 0;
  }
  sub dbh {
    my ($self, $dbh) = @_;
    if ($dbh) {
      $self->set_dbh($dbh);
    }
    if (!$self->get_dbh()) {
      $dbh = $self->util->dbh();
      $self->set_dbh($dbh);
    }
    return $self->get_dbh();
  }

  sub get_run_timeline_from_npg {

    my ($self) = @_;
    my $id_run = $self->get_id_run();

    print "getting run timeline data from npg for $id_run....\n" or croak "can't print\n";

    my $util  = $self->util();
    my $npg_api_util = $util->get_npg_api_util();
    $npg_api_util->base_uri($self->npg_uri());
    my $tries = $NUM_TRIES;

    while ($tries-- > 0) { # retry to get around network timeouts

      eval {

        my $npg_api_run_obj = npg::api::run->new({
             'id_run' => $id_run,
             'util'   =>$npg_api_util,
        });

        my $list = $npg_api_run_obj->run_statuses();


	      
	     foreach my $st (@{$list}){
           $self->check_one_status($st);
	     }

        if(!$self->get_start_time()){
          my $analysis_date = $self->get_analysis_date_from_db();
          if($analysis_date){
             $self->set_start_time($analysis_date);
          }
        }
	     $self->init();
	     if($self->get_id_run_timeline()){
	       $self->update();
	     }elsif($self->get_start_time() || $self->get_complete_time() || $self->get_end_time()){
	       $self->save_data();
	     }else{
	       print "No timeline data available, not saved\n" or croak "can't print\n";
	     }
        $tries = $TRY_SUCCESS;
        1;

      } or do {
        carp $EVAL_ERROR;
        print "failed to obtain runs -- retrying\n" or croak "can't print\n";
      };
    }

    if ($tries == 0) {
      croak 'too many retries -- quitting';
    }

    return 1;
  }
  sub check_one_status{
    my ($self, $st) = @_;
 
    my ($start_time, $end_time, $complete_time);
	 my $des = $st->description();
	 if($des eq 'run complete'){

	            $complete_time = $st->date();
	            $self->set_complete_time($complete_time);

	 }elsif($des eq 'run mirrored'
	                 || $des eq 'run cancelled'
	                 || $des eq 'run quarantined'
	                 || $des eq 'data discarded'
	                 || $des eq 'run stopped early'
	                 ){

	            $end_time = $st->date();
	            $self->set_end_time($end_time);
	 }elsif($des eq 'run pending'){

	            $start_time = $st->date();
	            $self->set_start_time($start_time);
	 }
	 return 1;
  }
  sub save_data {
    my ($self) = @_;
    
    my $query = q{INSERT into run_timeline(id_run, start_time, complete_time, end_time) VALUES (?,?,?,?)};

    my $dbh = $self->dbh();
    eval {
      $dbh->do($query,
                {}, $self->get_id_run(), $self->get_start_time, $self->get_complete_time, $self->get_end_time);
      $dbh->commit();
      1;
    } or do {
      if ($EVAL_ERROR =~ /Duplicate\ entry/xms) {
        carp "$EVAL_ERROR - continuing anyway";
      } else {
        $dbh->rollback();
        croak $EVAL_ERROR;
      }
    };

    return 1;
  }
  
  sub update {
    my ($self) = @_;
    
    my $query = q{update run_timeline set start_time = ?, complete_time = ?, end_time = ? where id_run_timeline = ?};

    my $dbh = $self->dbh();
    eval {
      $dbh->do($query,
                {}, $self->get_start_time, $self->get_complete_time, $self->get_end_time, $self->get_id_run_timeline);
      $dbh->commit();
      1;
    } or do {
      if ($EVAL_ERROR =~ /Duplicate\ entry/xms) {
        carp "$EVAL_ERROR - continuing anyway";
      } else {
        $dbh->rollback();
        croak $EVAL_ERROR;
      }
    };

    return 1;
  }

  sub get_runlist_from_npg {
    my ($self) = @_;

    print "getting a list of runs from npg....\n" or croak "can't print\n";
    $self->set_run_list_all([]);
    
    my $util  = $self->util();
    my $npg_api_util = $util->get_npg_api_util();
    $npg_api_util->base_uri($self->npg_uri());
    my $tries = $NUM_TRIES;
    while ($tries-- > 0) { # retry to get around network timeouts

      eval {

        my $npg_api_run_obj = npg::api::run->new({
              'util'   =>$npg_api_util,
        });
        my $run_list = $npg_api_run_obj->runs();
        foreach my $run (@{$run_list}){
           push @{$self->get_run_list_all()}, $run->id_run();
        }
        my $num_runs_total = scalar @{$run_list};
        print "there are $num_runs_total runs in total from npg\n" or croak "can't print\n";
        $tries = $TRY_SUCCESS;
        1;

      } or do {
        carp $EVAL_ERROR;
        carp 'failed to obtain runs -- retrying';
      };
    }

    if ($tries == 0) {
      croak 'too many retries -- quitting';
    }
    return 1;
  }
  
  sub get_runlist_done_from_db{
    my ($self) = @_;
    print "getting a list of runs alreay in the database....\n" or croak "can't print\n";
    $self->set_run_list_done({});

    my $query = q{SELECT id_run FROM run_timeline
                  WHERE start_time IS NOT NULL
                  AND complete_time IS NOT NULL
                  AND end_time IS NOT NULL};

    my $dbh = $self->dbh();
    my $ref;
    eval {
      $ref = $dbh->selectall_arrayref($query, {});
      my $num_runs_in_db = scalar @{$ref};
      if($num_runs_in_db) {
        foreach my $row (@{$ref}){
           $self->get_run_list_done()->{$row->[0]} = 1;
        }
      }
      print "There are $num_runs_in_db in the database with timeline date\n" or croak "can't print\n";
      1;
    } or do {
        croak $EVAL_ERROR;
    };
    return 1;
  }

  sub save_all_runs_date{
    my ($self) = @_;
    my $count_runs_done = 0;

    $self->get_runlist_from_npg();
    $self->get_runlist_done_from_db();
 
    foreach my $id_run (@{$self->get_run_list_all()}){
      if(!$self->get_run_list_done->{$id_run} && $id_run > $MINIMUM_ID_RUN){
          my $util  = $self->util();
          my $run_timeline_obj = npg_qc::api::run_timeline->new({util   => $util});
          $run_timeline_obj->id_run($id_run);
  
          eval{
            $run_timeline_obj->get_run_timeline_from_npg();
            $count_runs_done ++;
            1;
          } or do {
            croak $EVAL_ERROR;
          };
        }
          
      }
      print "There are $count_runs_done runs saved timeline data from npg\n" or croak "can't print\n";
      return 1;
  }
}

1;
__END__

=head1 NAME

npg_qc::api::run_timeline

=head1 VERSION

$LastChangedRevision: 4437 $

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 get_analysis_date_from_db

=head2 save_all_runs_date

=head2 get_runlist_done_from_db

=head2 get_runlist_from_npg

=head2 update

=head2 check_one_status 

=head2 save_data

=head2 get_run_timeline_from_npg

=head2 init

=head2 id_run

=head2 dbh

=head2 fields

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

strict
warnings
Carp
English
npg_qc::api::base


=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Guoying Qi, E<lt>gq1@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2008 GRL, by Guoying Qi

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.4 or,
at your option, any later version of Perl 5 you may have available.

=cut
