#########
# Author:        ajb
# Maintainer:    $Author: gq1 $
# Created:       2008-06-11
# Last Modified: $Date: 2009-01-22 14:59:44 +0000 (Thu, 22 Jan 2009) $
# Id:            $Id: util.pm 4047 2009-01-22 14:59:44Z gq1 $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/trunk/lib/npg_qc/api/util.pm $
#
package npg_qc::api::util;
use base qw(Class::Accessor);
use strict;
use warnings;
use Class::Std;
use HTTP::Request::Common;
use LWP::UserAgent;
use XML::LibXML;
use npg::api::util;
use Carp;
use Readonly;
use DBI;
use English qw{-no_match_vars};
use Config::IniFiles;

our $VERSION  = do { my ($r) = q$LastChangedRevision: 4047 $ =~ /(\d+)/mxs; $r; };
our $DEBUG    = 0;

Readonly our $TIMEOUT => 60;

{
  ## no critic (ProhibitUnusedVariables)
  my %base_uri_of  :ATTR( init_arg => 'base_uri',     :get<base_uri>,     :set<base_uri> );
  my %npg_uri_of   :ATTR( init_arg => 'npg_uri',      :get<npg_uri>,      :set<npg_uri> );
  my %useragent_of :ATTR( init_arg => 'useragent',    :get<useragent>,    :set<useragent> );
  my %parser_of    :ATTR( init_arg => 'parser',       :get<parser>,       :set<parser> );
  my %npg_api_util :ATTR( init_arg => 'npg_api_util', :get<npg_api_util>, :set<npg_api_util> );
  my %dbh_of       :ATTR( :get<dbh>,    :set<dbh> );
  my %config_of    :ATTR( :get<config>, :set<config> );
  ## use critic

  sub BUILD {
    my ($self, @args) = @_;
    $args[1]->{base_uri}     || $self->set_base_uri('http://intweb.sanger.ac.uk/cgi-bin/prodsoft/npg_qc/npg_qc/');
    $args[1]->{npg_uri}      || $self->set_npg_uri('http://intweb.sanger.ac.uk/perl/prodsoft/npg/npg/');
    $args[1]->{parser}       || $self->set_parser(XML::LibXML->new());
    $args[1]->{npg_api_util} || $self->set_npg_api_util(npg::api::util->new());
    return;
  }

  sub base_uri {
    my ($self, $base_uri) = @_;
    if ($base_uri) {
      $self->set_base_uri($base_uri);
    }
    return $self->get_base_uri();
  }

  sub npg_uri {
    my ($self, $npg_uri) = @_;
    if ($npg_uri) {
      $self->set_npg_uri($npg_uri);
    }
    return $self->get_npg_uri();
  }

  sub useragent {
    my $self = shift;

    if(!$self->get_useragent()) {
      my $ua = LWP::UserAgent->new();
      $ua->agent("npg_qc::api::util $VERSION");
      $ua->timeout($TIMEOUT);
      $ua->env_proxy();
      $self->set_useragent($ua);
    }
    return $self->get_useragent();
  }

  sub parser {
    my $self = shift;
    $self->get_parser() || $self->set_parser(XML::LibXML->new());
    return $self->get_parser();
  }

  sub get {
    my ($self, $uri, $args) = @_;

    my $accept = $uri =~ /json/xms ? 'text/json'
               :                     'text/xml'
               ;
    my $req = GET $uri, 'Accept' => $accept, @{$args||[]};
    return $self->useragent->request($req);
  }

  sub post {
    my ($self, $uri, $args) = @_;
    my $req = POST $uri, 'Accept' => 'text/xml', @{$args||[]};
    return $self->useragent->request($req);
  }

  sub dbh {
    my ($self, $dbh) = @_;
    if ($dbh) {
      $self->set_dbh($dbh);
    }
    if (!$self->get_dbh()) {
      eval {
        my $dbname = $self->dbname();
	my $dbhost = $self->dbhost();
	my $dbuser = $self->dbuser();
	my $dbpass = $self->dbpass();
	my $dbport = $self->dbport();
        $dbh = DBI->connect(
                          "DBI:mysql:database=$dbname;host=$dbhost;port=$dbport",
                          "$dbuser",
                          "$dbpass",
                          {
                           RaiseError => 1,
                           PrintError => 0,
                           PrintWarn  => 0,
                           AutoCommit => 0,
                          });
        $self->set_dbh($dbh);
	1;
      } or do {
        croak 'unable to get a dbh - '.$EVAL_ERROR;
      };
    }
    return $self->get_dbh();
  }

  sub configpath {
    return 'data/config.ini';
  }


  sub config {
    my $self       = shift;
    my $configpath = $self->configpath() || q();
    my $dtconfigpath;

    if(!$self->get_config()) {
      ($dtconfigpath) = $configpath =~ m{([a-z\d_/\.\-]+)}mixs;
      $dtconfigpath ||= q();

      if($dtconfigpath ne $configpath) {
        croak qq(Failed to detaint configpath: '$configpath');
      }

      if(!-e $dtconfigpath) {
        croak qq(No such file: $dtconfigpath);
      }

      $self->set_config(Config::IniFiles->new(-file => $dtconfigpath));
    }

    if(!$self->get_config()) {
      croak qq(No configuration available:\n). join q(, ), @Config::IniFiles::errors; ##no critic (ProhibitPackageVars)
    }

    return $self->get_config();
  }

  sub dbsection {
    return $ENV{dev} || 'live';
  }

  sub dbname { my $self = shift; return $self->config->val($self->dbsection(), 'dbname') || q(npg); }
  sub dbhost { my $self = shift; return $self->config->val($self->dbsection(), 'dbhost') || q(localhost); }
  sub dbport { my $self = shift; return $self->config->val($self->dbsection(), 'dbport') || q(3306); }
  sub dbuser { my $self = shift; return $self->config->val($self->dbsection(), 'dbuser') || q(root); }
  sub dbpass { my $self = shift; return $self->config->val($self->dbsection(), 'dbpass') || q(); }
}

1;
__END__

=head1 NAME

npg_qc::api::util

=head1 VERSION

$LastChangedRevision: 4047 $

=head1 SYNOPSIS

  my $oUtil = npg_qc::api::util->new({
    useragent => $oUserAgent,
    base_uri  => $sBaseURI,
    parser    => $oXmlParser,
  });

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 BUILD - Additional functions to constructor, sets default uri and parser if not provided

=head2 base_uri - gets and sets base uri for this resource

  my $sURI = $oDerivedObject->base_uri();
  my $sNEW_URI = $oDerivedObject->base_uri($new_uri);

=head2 npg_uri - gets and sets npg uri for this resource

  my $sURI = $oUtil->npg_uri();
  my $sNEW_URI = $oUil->npg_uri($new_uri);

=head2 useragent - accessor to obtain LWP::UserAgent object

  my $oUserAgent = $oDerivedObject->useragent();

=head2 parser - accessor to obtain Xml_Parser object stored, or provide libxml2 XML::LibXML parser object by default

  my $oLibXMLParser = $oDerivedObject->parser();

=head2 get - HTTP GET with additional Accept: text/xml header

  my $oHTTPResponse = $oUtil->get($sURI, $arHTTPRequestHeaders);

=head2 post - HTTP POST with additional Accept: text/xml header

  my $oHTTPResponse = $oUtil->post($sURI, $arHTTPRequestHeaders);

=head2 dbh - accessor to get/store/create a database handle for npg_qc

=head2 configpath - returns the path from the current level to the data/config.ini file

=head2 config - get and read in the config.ini file

=head2 dbname - the attributes expected to produce a database connection handle
=head2 dbhost - read in from the config.ini file
=head2 dbport - or defaults given
=head2 dbuser - (given defaults should not produce
=head2 dbpass -  a connection to a specific database)

=head2 dbsection

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

Class::Std
Carp
LWP::UserAgent
HTTP::Request
XML::LibXML
Readonly
npg::api:util

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Andy Brown, E<lt>ajb@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2008 GRL, by Andy Brown

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.4 or,
at your option, any later version of Perl 5 you may have available.

=cut
