#########
# Author:        ajb
# Maintainer:    $Author: gq1 $
# Created:       2008-06-10
# Last Modified: $Date: 2009-01-21 16:02:20 +0000 (Wed, 21 Jan 2009) $
# Id:            $Id: swift_all_intensities_per_cycle_per_base.pm 4034 2009-01-21 16:02:20Z gq1 $
# Source:        $Source: /repos/cvs/webcore/SHARED_docs/cgi-bin/docrep,v $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/trunk/lib/npg_qc/model/swift_all_intensities_per_cycle_per_base.pm $
#

package npg_qc::model::swift_all_intensities_per_cycle_per_base;
use strict;
use warnings;
use base qw(npg_qc::model);
use npg_qc::model::run_tile;
use Statistics::Lite qw(:all);

our $VERSION = do { my ($r) = q$LastChangedRevision: 4034 $ =~ /(\d+)/mxs; $r; };

__PACKAGE__->mk_accessors(__PACKAGE__->fields());

sub fields {
  return qw(
            id_swift_all_intensities_per_cycle_per_base
            id_run_tile
            cycle
            a
            c
            g
            t
          );
}

sub run_tile {
  my $self  = shift;
  my $pkg   = 'npg_qc::model::run_tile';
  return $pkg->new({
		    'util' => $self->util(),
		    'id_run_tile' => $self->id_run_tile(),
		   });
}

sub swift_all_intensities_by_id_run {
  my ($self, $id_run) = @_;
  my $pkg = ref$self;
  my $query = qq{SELECT @{[join q(, ), map { "saibir.$_ AS $_" }  $pkg->fields()]}, rt.tile AS tile, rt.position AS position
                 FROM   @{[$pkg->table]} saibir,
                        run_tile rt
                 WHERE  rt.id_run = ?
                 AND    rt.id_run_tile = saibir.id_run_tile
                 ORDER BY rt.position, rt.tile, saibir.cycle};
  return $self->gen_getarray($pkg, $query, $id_run);
}

sub average_intensities_over_cycle {
  my ($self, $id_run) = @_;
  my $lanes = {};
  foreach my $tile (@{$self->swift_all_intensities_by_id_run($id_run)}) {
    my $cycle = sprintf '%02d', $tile->cycle();
    push @{$lanes->{$tile->{position}}->{$cycle}}, $tile;
  }
  my $return_array = [];
  foreach my $position (sort keys %{$lanes}) {
    foreach my $cycle (sort keys %{$lanes->{$position}}) {
      my (@a, @c, @g, @t);
      foreach my $tile (@{$lanes->{$position}->{$cycle}}) {
        push @a, $tile->{a};
        push @c, $tile->{c};
        push @g, $tile->{g};
        push @t, $tile->{t};
      }
      push @{$return_array}, [$position, $cycle, mean(@a), mean(@c), mean(@g), mean(@t)];
    }
  }
  return $return_array;
}

1;
__END__
=head1 NAME

npg_qc::model::swift_all_intensities_per_cycle_per_base

=head1 VERSION

$LastChangedRevision: 4034 $

=head1 SYNOPSIS

  my $oSwiftAllIntensitiesPerCyclePerBase = npg_qc::model::->new({util => $util});

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 fields - return array of fields, first of which is the primary key

  my $aFields = $oSwiftAllIntensitiesPerCyclePerBase->fields();

=head2 run_tile - returns run_tile object that this object belongs to

  my $oRunTile = $oSwiftAllIntensitiesPerCyclePerBase->run_tile();

=head2 swift_all_intensities_by_id_run - returns an array of objects relating to the given run, including position and tile no.

  my $aSwiftAllIntensitiesByIdRun = $oSwiftAllIntensitiesPerCyclePerBase->swift_all_intensities_by_id_run($id_run);

=head2 average_intensities_over_run - returns an array lanes, with the average intensities for each called base over the tiles

  my $aAverageIntensitiesOverRun = $oSwiftAllIntensitiesPerCyclePerBase->average_intensities_over_run($id_run);

=head2 average_intensities_over_cycle - returns an array of all the cycles, with the average intensities calulated for each base over the cycle

  my $aAverageIntensitiesOverCycle = $oSwiftAllIntensitiesPerCyclePerBase->average_intensities_over_cycle($id_run);

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

strict
warnings
npg_qc::model
npg_qc::model::run_tile

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Andy Brown, E<lt>ajb@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2008 GRL, by Andy Brown

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.4 or,
at your option, any later version of Perl 5 you may have available.

=cut
