#########
# Author:        ajb
# Maintainer:    $Author: gq1 $
# Created:       2008-06-10
# Last Modified: $Date: 2009-01-21 16:02:20 +0000 (Wed, 21 Jan 2009) $
# Id:            $Id: swift_pf_errors_per_cycle.pm 4034 2009-01-21 16:02:20Z gq1 $
# Source:        $Source: /repos/cvs/webcore/SHARED_docs/cgi-bin/docrep,v $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/trunk/lib/npg_qc/model/swift_pf_errors_per_cycle.pm $
#

package npg_qc::model::swift_pf_errors_per_cycle;
use strict;
use warnings;
use base qw(npg_qc::model);
use npg_qc::model::run_tile;
use Statistics::Lite qw(:all);
use Readonly;

our $VERSION = do { my ($r) = q$LastChangedRevision: 4034 $ =~ /(\d+)/mxs; $r; };

Readonly our $PERCENTAGE => 100;
Readonly our $MAX_ERROR_PERCENTAGE => 20; # any error greater than this is capped for graphing purposes

__PACKAGE__->mk_accessors(__PACKAGE__->fields());

sub fields {
  return qw(
            id_swift_pf_errors_per_cycle
            id_run_tile
            cycle
            errors
          );
}

sub run_tile {
  my $self  = shift;
  my $pkg   = 'npg_qc::model::run_tile';
  return $pkg->new({
		    'util' => $self->util(),
		    'id_run_tile' => $self->id_run_tile(),
		   });
}

sub swift_pf_errors_by_id_run {
  my ($self, $id_run) = @_;
  my $pkg = ref$self;
  my $query = qq{SELECT @{[join q(, ), map { "pepc.$_ AS $_" }  $pkg->fields()]}, rt.tile AS tile, rt.position AS position
                 FROM   @{[$pkg->table]} pepc,
                        run_tile rt
                 WHERE  rt.id_run = ?
                 AND    rt.id_run_tile = pepc.id_run_tile
                 ORDER BY rt.position, rt.tile};
  return $self->gen_getarray($pkg, $query, $id_run);
}

sub total_errors_over_cycle {
  my ($self, $id_run, $cycle_count, $agg_swift_report) = @_;
  my $lanes = {};
  foreach my $tile (@{$self->swift_pf_errors_by_id_run($id_run)}) {
    my $cycle = sprintf '%02d', $tile->cycle();
    push @{$lanes->{$tile->{position}}->{$cycle}}, $tile;
  }
  my $return_array = [];
  foreach my $position (sort keys %{$lanes}) {
    my $perc_maker = $PERCENTAGE*$cycle_count/$agg_swift_report->[$position-1]->[1];
    foreach my $cycle (sort keys %{$lanes->{$position}}) {
      my @errors;
      foreach my $tile (@{$lanes->{$position}->{$cycle}}) {
        push @errors, $tile->{errors};
      }
      my $sum_errors = sum(@errors);
      my $perc_error = sprintf '%.02f', ($sum_errors*$perc_maker);
      if ($perc_error > $MAX_ERROR_PERCENTAGE) {
        $perc_error = sprintf '%.02f', $MAX_ERROR_PERCENTAGE;
      }
      push @{$return_array}, [$position, $cycle, $sum_errors, $perc_error];
    }
  }
  return $return_array;
}

sub errors_as_percentage {
  my ($self, $cycle_count) = @_;
  my $swift_report = $self->run_tile->swift_reports()->[0];
  my $percentage = $self->errors()*$PERCENTAGE*$cycle_count/$swift_report->pf_bases();
  if ($percentage > $MAX_ERROR_PERCENTAGE) {
    return $MAX_ERROR_PERCENTAGE;
  }
  return $percentage;
}

1;
__END__
=head1 NAME

npg_qc::model::swift_pf_errors_per_cycle

=head1 VERSION

$LastChangedRevision: 4034 $

=head1 SYNOPSIS

  my $oSwiftPFErrorsPerCycle = npg_qc::model::swift_pf_errors_per_cycle->new({util => $util});

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 fields - return array of fields, first of which is the primary key

  my $aFields = $oSwiftPFErrorsPerCycle->fields();

=head2 run_tile - returns run_tile object that this object belongs to

  my $oRunTile = $oSwiftPFErrorsPerCycle->run_tile();

=head2 swift_pf_errors_by_id_run - returns array of the pf errors for an id_run

  my $aSwiftPFErrorsBYIdRun = $oSwiftPFErrorsPerCycle->swift_pf_errors_by_id_run($id_run)

=head2 total_errors_over_cycle - returns an array of cycles, with the total errors and percentage errors for the run

  my $aTotalErrorsOverCycle = $oSwiftPFErrorsPerCycle->total_errors_over_cycle($id_run, $cycle_count, $agg_swift_report)

=head2 errors_as_percentage - returns the errors as a percentage of the total pf bases/cycle count

  my $sErrorsAsPercentage = $oSwiftPFErrorsPerCycle->errors_as_percentage($cycle_count);

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

strict
warnings
npg_qc::model
npg_qc::model::run_tile

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Andy Brown, E<lt>ajb@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2008 GRL, by Andy Brown

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.4 or,
at your option, any later version of Perl 5 you may have available.

=cut
