#########
# Author:        ajb
# Maintainer:    $Author: gq1 $
# Created:       2008-06-30
# Last Modified: $Date: 2009-01-21 16:02:20 +0000 (Wed, 21 Jan 2009) $
# Id:            $Id: swift_summary.pm 4034 2009-01-21 16:02:20Z gq1 $
# Source:        $Source: /repos/cvs/webcore/SHARED_docs/cgi-bin/docrep,v $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/trunk/lib/npg_qc/view/swift_summary.pm $
#

package npg_qc::view::swift_summary;
use strict;
use warnings;
use base qw(npg_qc::view);
use English qw{-no_match_vars};
use Carp;
use npg_qc::model::summary;
use Readonly;

our $VERSION = do { my ($r) = q$LastChangedRevision: 4034 $ =~ /(\d+)/mxs; $r; };

Readonly our $HEIGHT => 336;
Readonly our $WIDTH  => 504;
Readonly our $MAX_ERROR_PERCENTAGE => 20; # any error greater than this is capped for graphing purposes
Readonly our $X_LABEL_SKIP => 5;
Readonly our $THUMB_HEIGHT => 100;
Readonly our $THUMB_WIDTH  => 150;

sub list {
  my ($self) = @_;
  return $self->list_refactor();
}

sub add_ajax {
  my ($self) = @_;
  return $self->list_refactor();
}

sub list_ivc {
  my ($self) = @_;
  return $self->list_refactor();
}

sub list_errors {
  my ($self) = @_;
  return $self->list_refactor();
}

sub list_errors_tile {
  my ($self) = @_;
  return $self->list_refactor();
}

sub read_png {
  my ($self)    = @_;
  my $util      = $self->util();
  my $model     = $self->model();
  my $cgi       = $util->cgi();
  my $plot      = $cgi->param('plot');
  my $id_run    = $cgi->param('id_run');
  my $position  = $cgi->param('position');
  my $tile      = $cgi->param('tile');
  my $thumbnail = $cgi->param('thumb');
  my $graph_obj = npg_qc::graph->new({util => $util});
  my $title     = 'Swift '. ucfirst$plot . " for Run $id_run, Position $position";

  if ($tile) {
    $title .= ", Tile $tile";
  }

  # critic disabled here as unknown issue with the ','s in the hash refs
  my $args = $thumbnail eq 'true'    ? {
                                        height       => $THUMB_HEIGHT,
                                        width        => $THUMB_WIDTH,
                                        y_min_value  => 0,
                                        x_label_skip => $X_LABEL_SKIP,
                                       }
           : $plot =~ /all/xms       ? {
                                        x_label      => 'Cycle',
                                        y_label      => 'Intensity',
                                        legend       => [qw(a c g t)],
                                        title        => $title,
                                        height       => $HEIGHT,
                                        width        => $WIDTH,
                                        x_label_skip => $X_LABEL_SKIP,
                                        y_min_value  => 0,
                                       }
           : $plot =~ /pf_errors/xms ? {
                                        x_label      => 'Cycle',
                                        y_label      => 'Percentage',
                                        title        => $title,
                                        height       => $HEIGHT,
                                        width        => $WIDTH,
                                        y_min_value  => 0,
                                        y_max_value  => $MAX_ERROR_PERCENTAGE,
                                        x_label_skip => $X_LABEL_SKIP,
                                       }
           :                           {}
           ;

  if ($thumbnail eq 'true' && $plot =~ /pf_errors/xms) {
    $args->{y_max_value} = $MAX_ERROR_PERCENTAGE;
  }

  my $types = {
    all             => q{},
    call            => q{},
    pf_errors       => 'area',
    npf_errors      => 'area',
    tile_pf_errors  => 'area',
    tile_npf_errors => 'area',
  };

  $model->{id_run} = $id_run;
  my $data = $model->$plot($position, $tile);
  return $graph_obj->plotter($data, $args, $types->{$plot});
}

1;
__END__
=head1 NAME

npg_qc::view::swift_summary

=head1 VERSION

$LastChangedRevision: 4034 $

=head1 SYNOPSIS

  my $oSwiftSummaryView = npg_qc::view::swift_summary->new({util => $util});

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 new - inherited from base class, sets up object, passing into it the arguments given, and determining user

=head2 authorised - inherited from base class, only showing analysis data, at this time all are authorised

=head2 decor - sets decor to 0 if aspect is graph, else default

=head2 content_type - sets content_type to image/png if aspect is graph, else default

=head2 render - renders view, determining the aspect to view

=head2 list

=head2 list_ivc - handles rendering the ivc view

=head2 add_ajax

=head2 list_errors - handles rendering the errors per lane view

=head2 read_png - handles rendering a png image for an ivc view

=head2 list_errors_tile - handles rendering the errors per tile view

=head2 list_refactor - refactor of code which is common to all list view handlers (handles inserting the id_run into the model)

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES
strict
warnings
npg_qc::view
English
Carp

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Andy Brown, E<lt>ajb@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2008 GRL, by Andy Brown

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.4 or,
at your option, any later version of Perl 5 you may have available.

=cut
