########
# Author:        ajb
# Maintainer:    $Author: gq1 $
# Created:       2008-10-06
# Last Modified: $Date: 2008-12-12 12:07:02 +0000 (Fri, 12 Dec 2008) $
# Id:            $Id: 30-api-config_loader.t 3770 2008-12-12 12:07:02Z gq1 $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/trunk/t/30-api-config_loader.t $
#

use strict;
use warnings;
use Test::More tests => 11;
use English qw(-no_match_vars);
use lib qw{t};
use SangerPaths qw(core ensembl);
use t::api::util;
use npg::api::util;
use t::useragent;
use DBI; # ARGH!
use Carp;
use Perl6::Slurp;

our $VERSION = do { my ($r) = q$LastChangedRevision: 3770 $ =~ /(\d+)/mx; $r; };

use_ok('npg_qc::api::loader');
use_ok('npg_qc::api::loader::run_config');

my $ua   = t::useragent->new({is_success => 1, mock => {}});
my $npg_api_util = npg::api::util->new({useragent => $ua});
my $util = t::api::util->new({dbh => undef, npg_api_util => $npg_api_util, fixtures => 1, useragent => $ua, parser => 0, base_uri => 'http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/', npg_uri => 'http://localhost:8080/cgi-bin/prodsoft/npg/npg/'});
$util->load_fixtures();
my $test_dbh = $util->get_dbh();
isa_ok($test_dbh, q{DBI::db}, 'util has dbh');

my $config_test_xml = slurp 't/data/rendered/bustard_config.xml';
my $config_file_contents = q{# Auto-generated by; /nfs/team117/ts6/work/new-pipeline-dev/sanger-pipeline/branches/prerelease-7.0/bin/analyse_run v155
# At: 20080603-041506
# Goat executable: /software/solexa/src/SolexaPipeline/Goat/goat_pipeline.py
# Runfolders: /staging/IL21/analysis/080530_IL21_0812

# This is a every-20th-tile run.

# This is the first read (only) of a paired run.

EXPT_DIR .
EMAIL_SERVER mail.sanger.ac.uk
EMAIL_DOMAIN sanger.ac.uk
EMAIL_LIST ts6@sanger.ac.uk rmp@sanger.ac.uk cgb@sanger.ac.uk nw3@sanger.ac.uk jb1@sanger.ac.uk djt@sanger.ac.uk mdf@sanger.ac.uk cmc@sanger.ac.uk mjb@sanger.ac.uk ems@sanger.ac.uk lm4@sanger.ac.uk
WEB_DIR_ROOT http://sflogin.internal.sanger.ac.uk:9000/

ELAND_MULTIPLE_INSTANCES 8

SEQUENCE_FORMAT --fastq
QUALITY_FORMAT --symbolic

POST_RUN_COMMAND /nfs/team117/ts6/work/new-pipeline-dev/sanger-pipeline/branches/prerelease-7.0/bin/cleanup_run --verbose --nmake 4 --run 812 --lanes 1,2,3,5,6,7,8 > cleanup.out 2>&1

GENOME_DIR /software/solexa/Genomes/PhiX
GENOME_FILE phi_plus_SNPs.fa


1235678:ANALYSIS sequence
4:ANALYSIS default

1235678:USE_BASES Y36
4:USE_BASES Y36

};

my $loader = npg_qc::api::loader::run_config->new({ xml => $config_test_xml, util => $util });
isa_ok($loader, q{npg_qc::api::loader::run_config}, '$loader');
is($loader->util(), $util, '$loader has test util');
is($loader->get_xml(), $config_test_xml, 'xml stored in object ok');

my $before_save_rows;
eval { $before_save_rows = $test_dbh->selectall_arrayref(q{SELECT * FROM run_config}, {}); } or do { warn "Something went wrong"; };

$loader->dbh($test_dbh);
is($loader->dbh(), $test_dbh, 'test_dbh being used');

eval { $loader->read_xml(); } or do { warn "Something croaked\n\t$EVAL_ERROR\n"; };
is($EVAL_ERROR, q{}, 'config_xml read ok');
is($loader->get_id_run(), 812, 'id_run was obtained ok');
is($loader->get_config_text(), $config_file_contents, 'config_text was obtained ok');

#eval { $loader->save(); } or do { warn "Something croaked\n\t$EVAL_ERROR\n"; };
#is($EVAL_ERROR, q{}, 'no croak on row save');

my $rows;
eval { $rows = $test_dbh->selectall_arrayref(q{SELECT * FROM run_config}, {}); } or do { warn "Something went wrong"; };

my $before_count = scalar @{$before_save_rows};
my $after_count = scalar @{$rows};
is($after_count, $before_count + 1, 'one more row in database'); 
