#########
# Author:        ajb
# Maintainer:    $Author: gq1 $
# Created:       2008-06-12
# Last Modified: $Date: 2009-01-30 12:32:35 +0000 (Fri, 30 Jan 2009) $
# Id:            $Id: 30-api-main.t 4105 2009-01-30 12:32:35Z gq1 $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/trunk/t/30-api-main.t $
#

use strict;
use warnings;
use Test::More tests => 35;
use English qw(-no_match_vars);
use IO::Scalar;
use lib qw{t};
use SangerPaths qw(ensembl core badger trackingnpg tracking);
use t::api::util;
use npg::api::util;
use t::useragent;

our $VERSION = do { my ($r) = q$LastChangedRevision: 4105 $ =~ /(\d+)/mx; $r; };

use_ok('npg_qc::api::main');

my $file_root            = q{t/data/staging/IL21/analysis/080530_IL21_0812/Data/C2-37_Firecrest1.9.2_03-06-2008_single/Bustard1.9.2_03-06-2008_single/GERALD_03-06-2008_single/};
my $config_url           = $file_root.q{config.txt};
my $tile_all             = $file_root.q{s_1_0001_all.txt};
my $tile_score           = $file_root.q{s_4_0061_score.txt};
my $tile_rescore         = $file_root.q{s_4_0061_rescore.txt};
my $run_signal_means     = $file_root.q{Signal_Means.txt};

{
  my $ua_mock = {
    q{http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/create_xml}     => q{t/data/requests/cgi-bin/prodsoft/npg_qc/npg_qc/create_xml.xml},
    q{http://intweb.sanger.ac.uk/cgi-bin/prodsoft/npg/npg/run_status_dict}    => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict.xml},
    q{http://intweb.sanger.ac.uk/cgi-bin/prodsoft/npg/npg/run_status_dict/9}  => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict/9.xml},
    q{http://intweb.sanger.ac.uk/cgi-bin/prodsoft/npg/npg/run_status_dict/12} => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict/12.xml},
    q{http://intweb.sanger.ac.uk/cgi-bin/prodsoft/npg/npg/run_status_dict/17} => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict/17.xml},
    q{http://intweb.sanger.ac.uk/cgi-bin/prodsoft/npg/npg/run_status_dict/18} => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict/18.xml},
    q{http://intweb.sanger.ac.uk/cgi-bin/prodsoft/npg/npg/run_status_dict/19} => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict/19.xml},
    q{http://intweb.sanger.ac.uk/cgi-bin/prodsoft/npg/npg/run_status_dict/20} => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict/20.xml}, 

    q{http://intweb.sanger.ac.uk/perl/prodsoft/npg/npg/run_status_dict}    => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict.xml},
    q{http://intweb.sanger.ac.uk/perl/prodsoft/npg/npg/run_status_dict/9}  => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict/9.xml},
    q{http://intweb.sanger.ac.uk/perl/prodsoft/npg/npg/run_status_dict/12} => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict/12.xml},
    q{http://intweb.sanger.ac.uk/perl/prodsoft/npg/npg/run_status_dict/17} => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict/17.xml},
    q{http://intweb.sanger.ac.uk/perl/prodsoft/npg/npg/run_status_dict/18} => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict/18.xml},
    q{http://intweb.sanger.ac.uk/perl/prodsoft/npg/npg/run_status_dict/19} => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict/19.xml},
    q{http://intweb.sanger.ac.uk/perl/prodsoft/npg/npg/run_status_dict/20} => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict/20.xml},
    
    q{http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc//run_tile}      => q{t/data/requests/cgi-bin/prodsoft/npg_qc/npg_qc/run_tile.xml},
    q{http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc//summary}   => q{t/data/requests/cgi-bin/prodsoft/npg_qc/npg_qc/completed_run_summary.xml},
  };

  my $ua   = t::useragent->new({is_success => 1, mock => $ua_mock});
  my $npg_api_util = npg::api::util->new({useragent => $ua});
  my $util = t::api::util->new({dbh => 'dummy', npg_api_util => $npg_api_util, fixtures => 0, useragent => $ua, parser => 0, base_uri => 'http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/', npg_uri => 'http://intweb.sanger.ac.uk/cgi-bin/prodsoft/npg/npg/'});
  my $main = npg_qc::api::main->new({util => $util});

  $main->set_bustard_file_urls([]);
  push @{$main->get_bustard_file_urls()}, ($config_url, $tile_all, $tile_score, $tile_rescore, $run_signal_means);
  is(scalar@{$main->get_bustard_file_urls()}, 5, '5 uris loaded into $main->get_bustard_file_urls()');

  eval{ $main->retrieve_bustard_xml(); };
  is($EVAL_ERROR, q{}, 'no croak on $main->retrieve_bustard_xml()');
  is(scalar@{$main->get_bustard_output()}, 4, '4 xml outputs stored in $main->get_bustard_output()');
  ok($util->test_rendered($main->get_bustard_output()->[0], 't/data/rendered/bustard_config.xml'), '$main->get_bustard_output()->[0] is correct');
#test removed as no longer processing these files
#  ok($util->test_rendered($main->get_bustard_output()->[1], 't/data/rendered/tile_all.xml'), '$main->get_bustard_output()->[1] is correct');
  ok($util->test_rendered($main->get_bustard_output()->[1], 't/data/rendered/tile_score.xml'), '$main->get_bustard_output()->[2] is correct');
  ok($util->test_rendered($main->get_bustard_output()->[2], 't/data/rendered/tile_score_xml.xml'), '$main->get_bustard_output()->[3] is correct');
  ok($util->test_rendered($main->get_bustard_output()->[3], 't/data/rendered/signal_means.xml'), '$main->get_bustard_output()->[4] is correct');

  eval { $main->websend_bustard_xml(); };
  is($EVAL_ERROR, q{}, 'no croak on $mail->websend_bustard_xml()');
  is(scalar@{$main->get_response_objects()}, 4, '4 successful response objects');
  my $processed = $main->process_response_objects();
  isa_ok($processed, 'HASH', '$main->process_response_objects()');
  is($processed->{score}, 4, 'processed 4 response objects');

  eval{ $main->look_for_runs(); };
  is($EVAL_ERROR, q{}, '$main->look_for_runs() did not croak');
  isa_ok($main->get_runs(), 'ARRAY', '$main->look_for_runs() stored');
  is(scalar@{$main->get_runs()}, 6, '$main->look_for_runs() has correct number of runs returned in stored array');

  eval{ $main->bustard_runs_done(); };
  is($EVAL_ERROR, q{}, '$main->bustard_runs_done() did not croak');
  isa_ok($main->get_bustard_runs_done(), 'HASH', '$main->bustard_runs_done() stored');
  is(scalar keys%{$main->get_bustard_runs_done()}, 4, '$main->bustard_runs_done() has correct number of runs_done ids in stored hash');

  eval{ $main->process_all_bustard(); };
  like($EVAL_ERROR, qr{No\ files\ to\ process\ at\ this\ time\!}, '$main->process_all_bustard() croaked as no runs to process');

  my $has_runs_to_process;

  eval{ $has_runs_to_process = $main->retrieve_bustard_file_urls(); };
  is($EVAL_ERROR, q{}, '$main->retrieve_bustard_file_urls() did not croak');
  is($has_runs_to_process, 0, 'no files to process as all runs done');

  $ua_mock={q{http://intweb.sanger.ac.uk/cgi-bin/prodsoft/npg/npg/run_status_dict/9} => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict/9extra.xml},
  q{http://intweb.sanger.ac.uk/perl/prodsoft/npg/npg/run_status_dict/9} => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict/9extra.xml},
  };
  
  $main->set_file_root('t/data');
  $main->set_file_dir('staging/IL*/analysis/*run_no/Data/*Firecrest*_single/Bustard*/GERALD*/');
  $main->set_file_ext('*txt');
  $main->set_summary_dir('staging/IL*/outgoing/*run_no/Latest_Summary/Summary.htm');
  $main->set_fastqcheck_dir('staging/IL*/analysis/*run_no/Data/*Firecrest*_single/archive/*fastqcheck');

  eval{ $has_runs_to_process = $main->retrieve_bustard_file_urls(); };
  is($EVAL_ERROR, q{}, '$main->retrieve_bustard_file_urls() did not croak');
  is($has_runs_to_process, 0, 'no files to process, as 812 not full analysis');
  is(scalar@{$main->get_bustard_file_urls()}, 0, 'correct number of files found');
  is($main->get_bustard_file_urls()->[0], undef, q{no files in array});

  eval{ $main->process_all_bustard(); };
  like($EVAL_ERROR, qr{No\ files\ to\ process\ at\ this\ time!}, q{$main->process_all_bustard() croaked, as no runs to process});
}

{
  my $ua   = t::useragent->new({is_success => 0, mock => {}});
  my $util = t::api::util->new({dbh => 'dummy', fixtures => 0, useragent => $ua, parser => 0, base_uri => 0});
  my $main = npg_qc::api::main->new({util => $util});

  $main->set_bustard_file_urls([]);
  push @{$main->get_bustard_file_urls()}, ($config_url);
  $main->retrieve_bustard_xml();

  my $response = $main->websend_bustard_xml();
  is($response, 0, '$mail->websend_bustard_xml() is not success');
}


{
   my $ua_mock={
   q{http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/create_xml}     => q{t/data/requests/cgi-bin/prodsoft/npg_qc/npg_qc/create_xml.xml},
    q{http://intweb.sanger.ac.uk/cgi-bin/prodsoft/npg/npg/run_status_dict}    => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict.xml},
    q{http://intweb.sanger.ac.uk/cgi-bin/prodsoft/npg/npg/run_status_dict/12} => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict/12.xml},
    q{http://intweb.sanger.ac.uk/cgi-bin/prodsoft/npg/npg/run_status_dict/17} => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict/17.xml},
    q{http://intweb.sanger.ac.uk/cgi-bin/prodsoft/npg/npg/run_status_dict/18} => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict/18.xml},
    q{http://intweb.sanger.ac.uk/cgi-bin/prodsoft/npg/npg/run_status_dict/19} => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict/19.xml},
    q{http://intweb.sanger.ac.uk/cgi-bin/prodsoft/npg/npg/run_status_dict/20} => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict/20.xml},
    
        q{http://intweb.sanger.ac.uk/perl/prodsoft/npg/npg/run_status_dict}    => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict.xml},
    q{http://intweb.sanger.ac.uk/perl/prodsoft/npg/npg/run_status_dict/12} => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict/12.xml},
    q{http://intweb.sanger.ac.uk/perl/prodsoft/npg/npg/run_status_dict/17} => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict/17.xml},
    q{http://intweb.sanger.ac.uk/perl/prodsoft/npg/npg/run_status_dict/18} => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict/18.xml},
    q{http://intweb.sanger.ac.uk/perl/prodsoft/npg/npg/run_status_dict/19} => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict/19.xml},
    q{http://intweb.sanger.ac.uk/perl/prodsoft/npg/npg/run_status_dict/20} => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict/20.xml},
    
    q{http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc//run_tile}      => q{t/data/requests/cgi-bin/prodsoft/npg_qc/npg_qc/run_tile.xml},
    q{http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc//summary}   => q{t/data/requests/cgi-bin/prodsoft/npg_qc/npg_qc/completed_run_summary.xml},
     q{http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/cache_query}   => q{t/data/requests/cgi-bin/prodsoft/npg_qc/npg_qc/run_caching.xml},
     q{http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/run_graph}   => q{t/data/requests/cgi-bin/prodsoft/npg_qc/npg_qc/run_caching.xml},
   q{http://intweb.sanger.ac.uk/cgi-bin/prodsoft/npg/npg/run_status_dict/9} => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict/9_1829.xml},
  q{http://intweb.sanger.ac.uk/perl/prodsoft/npg/npg/run_status_dict/9} => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict/9_1829.xml},   
  };
  
  my $ua   = t::useragent->new({is_success => 1, mock => $ua_mock});
  my $npg_api_util = npg::api::util->new({useragent => $ua});
  my $util = t::api::util->new({dbh => undef, npg_api_util => $npg_api_util, fixtures => 1, useragent => $ua, parser => 0, base_uri => 'http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/', npg_uri => 'http://intweb.sanger.ac.uk/cgi-bin/prodsoft/npg/npg/'});
  my $main = npg_qc::api::main->new({util => $util});
  $util->load_fixtures();
  $main->set_file_root('t/data');
  $main->set_file_dir('staging/IL*/outgoing/*run_no/Data/*Firecrest*/Bustard*/GERALD*/');
  $main->set_file_ext('*txt');
  $main->set_signal_mean_dir('staging/IL*/outgoing/*run_no/Data/*Firecrest*/Bustard*/Signal_Means.txt');
  $main->set_score_dir('staging/IL*/outgoing/*run_no/Data/*Firecrest*/Bustard*/GERALD*/Temp/');
  $main->set_summary_dir('staging/IL*/outgoing/*run_no/Data/*Firecrest*/Bustard*/GERALD*/Summary.htm');
  $main->set_fastqcheck_dir('staging/IL*/outgoing/*run_no/Data/*Firecrest*/archive/*fastqcheck');
  
  my $has_runs_to_process;
  eval{ $has_runs_to_process = $main->retrieve_bustard_file_urls(); };
  is($EVAL_ERROR, q{}, '$main->retrieve_bustard_file_urls() did not croak');
  is($has_runs_to_process, 1, '1 file to process, as 1829 full analysis');


}

{
   my $ua_mock={
   q{http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/create_xml}     => q{t/data/requests/cgi-bin/prodsoft/npg_qc/npg_qc/create_xml.xml},
    q{http://intweb.sanger.ac.uk/cgi-bin/prodsoft/npg/npg/run_status_dict}    => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict.xml},
    q{http://intweb.sanger.ac.uk/cgi-bin/prodsoft/npg/npg/run_status_dict/12} => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict/12.xml},
    q{http://intweb.sanger.ac.uk/cgi-bin/prodsoft/npg/npg/run_status_dict/17} => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict/17.xml},
    q{http://intweb.sanger.ac.uk/cgi-bin/prodsoft/npg/npg/run_status_dict/18} => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict/18.xml},
    q{http://intweb.sanger.ac.uk/cgi-bin/prodsoft/npg/npg/run_status_dict/19} => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict/19.xml},
    q{http://intweb.sanger.ac.uk/cgi-bin/prodsoft/npg/npg/run_status_dict/20} => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict/20.xml},
    
        q{http://intweb.sanger.ac.uk/perl/prodsoft/npg/npg/run_status_dict}    => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict.xml},
    q{http://intweb.sanger.ac.uk/perl/prodsoft/npg/npg/run_status_dict/12} => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict/12.xml},
    q{http://intweb.sanger.ac.uk/perl/prodsoft/npg/npg/run_status_dict/17} => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict/17.xml},
    q{http://intweb.sanger.ac.uk/perl/prodsoft/npg/npg/run_status_dict/18} => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict/18.xml},
    q{http://intweb.sanger.ac.uk/perl/prodsoft/npg/npg/run_status_dict/19} => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict/19.xml},
    q{http://intweb.sanger.ac.uk/perl/prodsoft/npg/npg/run_status_dict/20} => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict/20.xml},
    
    q{http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc//run_tile}      => q{t/data/requests/cgi-bin/prodsoft/npg_qc/npg_qc/run_tile.xml},
    q{http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc//summary}   => q{t/data/requests/cgi-bin/prodsoft/npg_qc/npg_qc/completed_run_summary.xml},
     q{http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/cache_query}   => q{t/data/requests/cgi-bin/prodsoft/npg_qc/npg_qc/run_caching.xml},
     q{http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/run_graph}   => q{t/data/requests/cgi-bin/prodsoft/npg_qc/npg_qc/run_caching.xml},
   q{http://intweb.sanger.ac.uk/cgi-bin/prodsoft/npg/npg/run_status_dict/9} => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict/9_1873.xml},
  q{http://intweb.sanger.ac.uk/perl/prodsoft/npg/npg/run_status_dict/9} => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict/9_1873.xml},   
  };
  
  my $ua   = t::useragent->new({is_success => 1, mock => $ua_mock});
  my $npg_api_util = npg::api::util->new({useragent => $ua});
  my $util = t::api::util->new({dbh => undef, npg_api_util => $npg_api_util, fixtures => 1, useragent => $ua, parser => 0, base_uri => 'http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/', npg_uri => 'http://intweb.sanger.ac.uk/cgi-bin/prodsoft/npg/npg/'});
  my $main = npg_qc::api::main->new({util => $util});
  $util->load_fixtures();
  $main->set_file_root('t/data');
  $main->set_file_dir('staging/IL28/outgoing/*run_no/Data/*Firecrest*/Bustard*/GERALD*/');
  $main->set_file_ext('*txt');
  
  $main->set_signal_mean_dir('staging/IL28/outgoing/*run_no/Data/*Firecrest*/Bustard*/Signal_Means.txt');
  $main->set_score_dir('staging/IL28/outgoing/*run_no/Data/*Firecrest*/Bustard*/GERALD*/Temp/');
  $main->set_summary_dir('staging/IL28/outgoing/*run_no/Data/*Firecrest*/Bustard*/GERALD*/Summary.htm');
  $main->set_fastqcheck_dir('staging/IL28/outgoing/*run_no/Data/*Firecrest*/archive/*fastqcheck');
  
  $main->set_matrix_dir('staging/IL28/outgoing/*run_no/Data/*Firecrest*/Matrix/s_*_*_matrix.txt');
  $main->set_offset_dir('staging/IL28/outgoing/*run_no/Data/*Firecrest*/Offsets/s_*_offsets.txt');

  my $has_runs_to_process;
  eval{ $has_runs_to_process = $main->retrieve_bustard_file_urls(); };
  is($EVAL_ERROR, q{}, '$main->retrieve_bustard_file_urls() 1873 did not croak');
  is($has_runs_to_process, 1, '1 file to process, as 1873 full analysis');


}

{   my $ua_mock={ };
  
  my $ua   = t::useragent->new({is_success => 1, mock => $ua_mock});
  my $npg_api_util = npg::api::util->new({useragent => $ua});
  my $util = t::api::util->new({dbh => undef, npg_api_util => $npg_api_util, fixtures => 0, useragent => $ua, parser => 0, base_uri => 'http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/', npg_uri => 'http://intweb.sanger.ac.uk/cgi-bin/prodsoft/npg/npg/'});
  my $main = npg_qc::api::main->new({util => $util});

  my $run_folder_dirs = $main->get_run_folder_from_summary_link('t/data','staging/IL*/outgoing/*1873' );
  is($run_folder_dirs->[0], 't/data/staging/IL28/outgoing/081128_IL28_1873', 'correct run folder');

  is($run_folder_dirs->[1], 't/data/staging/IL28/outgoing/081128_IL28_1873/Data/C1-72_Firecrest1.9.5_03-12-2008_auto', 'correct run folder firecrest');

  is($run_folder_dirs->[2], 't/data/staging/IL28/outgoing/081128_IL28_1873/Data/C1-72_Firecrest1.9.5_03-12-2008_auto/Bustard1.9.5_03-12-2008_auto', 'correct run folder bustard');
  
  is($run_folder_dirs->[3], 't/data/staging/IL28/outgoing/081128_IL28_1873/Data/C1-72_Firecrest1.9.5_03-12-2008_auto/Bustard1.9.5_03-12-2008_auto/GERALD_03-12-2008_auto/', 'correct run folder gerald'); 
}


__END__
runs now loading directly into database, so the following tests are not applicable anymore
{
  my $ua_mock = {
    q{http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/create_xml}     => q{t/data/requests/cgi-bin/prodsoft/npg_qc/npg_qc/create_xml.xml},
    q{http://intweb.sanger.ac.uk/cgi-bin/prodsoft/npg/npg/run_status_dict}    => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict.xml},
    q{http://intweb.sanger.ac.uk/cgi-bin/prodsoft/npg/npg/run_status_dict/9}  => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict/9additional.xml},
    q{http://intweb.sanger.ac.uk/cgi-bin/prodsoft/npg/npg/run_status_dict/12} => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict/12.xml},
    q{http://intweb.sanger.ac.uk/cgi-bin/prodsoft/npg/npg/run_status_dict/17} => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict/17.xml},
    q{http://intweb.sanger.ac.uk/cgi-bin/prodsoft/npg/npg/run_status_dict/18} => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict/18.xml},
    q{http://intweb.sanger.ac.uk/cgi-bin/prodsoft/npg/npg/run_status_dict/19} => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict/19.xml},
    q{http://intweb.sanger.ac.uk/cgi-bin/prodsoft/npg/npg/run_status_dict/20} => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict/20.xml},
    
        q{http://intweb.sanger.ac.uk/perl/prodsoft/npg/npg/run_status_dict}    => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict.xml},
    q{http://intweb.sanger.ac.uk/perl/prodsoft/npg/npg/run_status_dict/9}  => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict/9additional.xml},
    q{http://intweb.sanger.ac.uk/perl/prodsoft/npg/npg/run_status_dict/12} => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict/12.xml},
    q{http://intweb.sanger.ac.uk/perl/prodsoft/npg/npg/run_status_dict/17} => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict/17.xml},
    q{http://intweb.sanger.ac.uk/perl/prodsoft/npg/npg/run_status_dict/18} => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict/18.xml},
    q{http://intweb.sanger.ac.uk/perl/prodsoft/npg/npg/run_status_dict/19} => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict/19.xml},
    q{http://intweb.sanger.ac.uk/perl/prodsoft/npg/npg/run_status_dict/20} => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict/20.xml},
    
    q{http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc//run_tile}      => q{t/data/requests/cgi-bin/prodsoft/npg_qc/npg_qc/run_tile.xml},
    q{http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc//summary}   => q{t/data/requests/cgi-bin/prodsoft/npg_qc/npg_qc/completed_run_summary.xml},
     q{http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/cache_query}   => q{t/data/requests/cgi-bin/prodsoft/npg_qc/npg_qc/run_caching.xml},
     q{http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/run_graph}   => q{t/data/requests/cgi-bin/prodsoft/npg_qc/npg_qc/run_caching.xml},
  };
  my $ua   = t::useragent->new({is_success => 1, mock => $ua_mock});
  my $npg_api_util = npg::api::util->new({useragent => $ua});
  my $util = t::api::util->new({dbh => 'dummy', npg_api_util => $npg_api_util, fixtures => 0, useragent => $ua, parser => 0, base_uri => 'http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/', npg_uri => 'http://localhost:8080/cgi-bin/prodsoft/npg/npg/'});
  my $main = npg_qc::api::main->new({util => $util});

  $main->set_file_root('t/data');
  $main->set_file_dir('staging/IL*/analysis/*run_no/Data/*Firecrest*_auto/Bustard*/GERALD*/');
  $main->set_file_ext('*txt');
  $main->set_summary_dir('staging/IL*/outgoing/*run_no/Latest_Summary/Summary.htm');
  $main->set_fastqcheck_dir('staging/IL*/analysis/*run_no/Data/*Firecrest*_auto/archive/*fastqcheck');

  my $has_runs_to_process;

  eval{ $has_runs_to_process = $main->retrieve_bustard_file_urls(); };
  
  is($EVAL_ERROR, q{}, 'run 956 - $main->retrieve_bustard_file_urls() did not croak');
  is($has_runs_to_process, 1, 'run 956 - files to process as not all runs done');
  is(scalar@{$main->get_bustard_file_urls()}, 0, 'file_urls cleared down after processing');
  is($main->get_bustard_file_urls()->[0], undef, q{$main->get_bustard_file_urls()->[0] is correct (undef)});
  is(scalar@{$main->get_bustard_output()}, 0, 'correct number of xml strings  found');
  is($main->get_response_objects(), undef, 'response objects cleared after processing found');
  is(scalar@{$main->get_bustard_output()}, 0, 'correct number of xml strings  found');
  is($main->get_response_objects(), undef, 'response objects cleared after processing found');
  
}
1;
