#########
# Author:        gq1
# Maintainer:    $Author: gq1 $
# Created:       2009-02-27
# Last Modified: $Date: 2009-03-20 10:55:56 +0000 (Fri, 20 Mar 2009) $
# Id:            $Id: 30-api-run_recipe.t 4573 2009-03-20 10:55:56Z gq1 $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/trunk/t/30-api-run_recipe.t $
#

use strict;
use warnings;
use Test::More tests => 21;
use English qw(-no_match_vars);
use IO::Scalar;
use lib ('t');
use SangerPaths qw(ensembl core badger trackingnpg tracking);
use t::api::util;
use t::useragent;

our $VERSION = do { my ($r) = q$LastChangedRevision: 4573 $ =~ /(\d+)/mxs; $r; };

use_ok('npg_qc::api::run_recipe');

{
  my $ua_mock = { };
  my $ua   = t::useragent->new({is_success => 1, mock => $ua_mock});
  my $npg_api_util = npg::api::util->new({useragent => $ua});
  my $util = t::api::util->new({dbh => undef, npg_api_util => $npg_api_util, fixtures => 1, useragent => $ua, parser => 0, base_uri => 'http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/', npg_uri => 'http://localhost:8080/cgi-bin/prodsoft/npg/npg/'});
  
  $util->load_fixtures();
  my $test_dbh = $util->get_dbh();
  isa_ok($test_dbh, q{DBI::db}, 'util has dbh');

  my $run_recipe_obj = npg_qc::api::run_recipe->new({util   => $util});
  
  isa_ok($run_recipe_obj, 'npg_qc::api::run_recipe', 'run_recipe object');
  
  $run_recipe_obj->read_run_recipe('t/data/staging/IL28/outgoing/090206_IL28_2216');
  
  is($run_recipe_obj->get_col(), 2, 'get correct number of columns');
  is($run_recipe_obj->get_cycle(), 37, 'get correct number of cycles');
  is($run_recipe_obj->get_lane(), 8, 'get correct number of lanes');
  is($run_recipe_obj->get_tile(), 100, 'get correct number of tiles');
  
  $run_recipe_obj->id_run(2216);
  
  eval{
    $run_recipe_obj->save_data();
    1;
  };
  
  is($EVAL_ERROR, q{}, 'no croak when saving data');
  
  eval{
    $run_recipe_obj->save_data();
    1;
  };
  
  is($EVAL_ERROR, q{}, 'no croak when re-saving data');
  
  $run_recipe_obj->set_cycle(54);
  eval{
    $run_recipe_obj->init();
    $run_recipe_obj->update();
    1;
  };
  
  is($EVAL_ERROR, q{}, 'no croak when updating data');
  
  $run_recipe_obj->get_runlist_done_from_db();

  is(scalar keys %{$run_recipe_obj->get_runlist_done()}, 4, "correct number of runs in db");
  
  $run_recipe_obj->set_file_root('t/data');
  $run_recipe_obj->set_file_outgoing('staging/IL*/outgoing/*_*_*/Recipe*.xml');
  $run_recipe_obj->set_file_analysis('staging/IL*/analysis/*_*_*/Recipe*.xml');
  
  my $num_runfolders = $run_recipe_obj->retrieve_runfolder_list();
  is($num_runfolders, 1, 'correct number of runfolders in the test staging directory');

  $run_recipe_obj->set_file_root('t/data');
  $run_recipe_obj->set_file_outgoing('staging/IL28/outgoing/*_*_*/Recipe*.xml');
  $run_recipe_obj->set_file_analysis('staging/IL28/analysis/*_*_*/Recipe*.xml');
  eval{
    $run_recipe_obj->process_all_runfolder();
    1;
  };
  is($EVAL_ERROR, q{}, 'no croak when process all rundolders');
}


{
  my $ua_mock = {q{http://intweb.sanger.ac.uk/perl/prodsoft/npg/npg/run/1881}      => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run/1881.xml} ,
                 q{http://intweb.sanger.ac.uk/perl/prodsoft/npg/npg/run/1895}      => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run/1895.xml}};
  my $ua   = t::useragent->new({is_success => 1, mock => $ua_mock});
  my $npg_api_util = npg::api::util->new({useragent => $ua});
  my $util = t::api::util->new({dbh => undef, npg_api_util => $npg_api_util, fixtures => 1, useragent => $ua, parser => 0, base_uri => 'http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/', npg_uri => 'http://localhost:8080/cgi-bin/prodsoft/npg/npg/'});
  
  $util->load_fixtures();
  my $test_dbh = $util->get_dbh();

  my $run_recipe_obj = npg_qc::api::run_recipe->new({util   => $util});
  
  eval{
    $run_recipe_obj->get_runlist_from_runtile();
    1;
  };
  is($EVAL_ERROR, q{}, 'no croak when get a list of runs from runtile table');
  is(scalar @{$run_recipe_obj->get_runlist_runtile()}, 5, 'get correct number of runs from run_tile');
  

  is($run_recipe_obj->paired_id_run(1881), 1866, 'paired_id_run sub');

  is($run_recipe_obj->paired_id_run_from_db(11), 10, 'correct paired_id_run from db');
    
  eval{
     $run_recipe_obj->save_one_run_data(97, 330, 8, 37);
     1;
  };

  is($EVAL_ERROR, q{}, 'no croak for save one run data');

  my $cycle_number;
  eval{
     $cycle_number = $run_recipe_obj->get_cycle_from_db(20);
     1;
  };
  is($EVAL_ERROR, q{}, 'no croak for get cycle number from DB');
  is($cycle_number, 54, 'correct cycle number from DB');

  $run_recipe_obj->set_runlist_runtile([[1881, 100, 8],[1895, 330, 8]] );
  $run_recipe_obj->set_runlist_done({1881=>1});
  eval{
    $run_recipe_obj->process_all_runs_from_runtile();
    1;
  };
  is($EVAL_ERROR, q{}, 'no croak for saving ');
     
}
1;
